c=======================================================================
c=======================================================================
c
c   ANISOTROPIC DIELECTRIC TENSOR PACKAGE          LPP/ERM-KMS
c
c  This package has been carefully tested on different computers, and
c  different versions (single precision, partially double precision and
c  fully double precision) have been written to meet the requirements
c  of these computers. It is your responsibility to choose the appropriate
c  version for your computer.
c
c  If you find any problem with this software, we would like to know about
c  it. Contact us by e-mail: M.Evrard@kfa-juelich.de or mevrard@ulb.ac.be
c
c  This is a  double precision version for use on Vax,IBM,and other
c  small word-length computers. Nevertheless, all the variables belonging
c  to the interface are received or sent back as single precision.
c
c=======================================================================
c=======================================================================
c
c   Summary
c   -------
c Package for the computation of the dielectric tensor of a plasma made
c of a maximum of IONMAX ions, with anisotropic maxwellians distributions.
c The plasma parameters are transmitted through the common /plasma_cmn/ and
c the dielectric tensor components for each species through the common
c /tensor_cmn/. The dielectric tensor can be calculated to infinite order,
c truncated to first order in Larmor radius or even estimated in the
c cold limit,depending on the switch model. To get the proper Kperp2
c root(s) of the dispersion equation, you use either the routines
c fastwave or n_root.
c
c=======================================================================
c=======================================================================
c
c    Package INTERFACE
c    -----------------
c
c   i) *** Public Variables ***
c
c   PLASMA VARIABLES are tranfered through common /plasma_cmn/ according to:
c   ----------------
c     integer nions      ! number of ion species
c     real npar          ! kpar/k0
c     real omega         ! twopi* RF_frequency [s-1]
c     real bmag          ! local magnetic field [T]
c     real dense         ! electron density [m-3]
c     real teper         ! perpendicular electron temperature [keV]
c     real tepar         ! parallell electron temperature [keV]
c     real awi(IONMAX)   ! ion atomic mass Ai
c     real qi(IONMAX)    ! ion charge      Zi
c     real densi(IONMAX) ! ion density [m-3]
c     real tiper(IONMAX) ! perpendicular ion temperature [keV]
c     real tipar(IONMAX) ! parallell ion temperature [keV]
c
c     common /plasma_cmn/ npar,omega,bmag,dense,teper,tepar,awi,qi,
c                         densi,tiper,tipar,nions
c
c   Those variables must be set to their local values before calling
c   this package.
c
c
c   SWITCHES chooses between the Cold (model<0), Truncated (model=0) or
c   --------                       Full Hot dielectric tensor (model>0)
c
c   The choice can be different for each specie.
c
c      integer model(0:IONMAX)
c      common /switches/model
c
c   DIELECTRIC TENSOR :
c   -----------------
c
c    K0,K1,K2,K3,K4,K5 are as described in Plasma Waves, D.G.Swanson,
c    (4.156ff) p.156,
c
c      EXCEPTED for K0 -> K0/nperp2, K4 and K5 -> K4/nperp,K5/nperp !!
c
c
c   DIELECTRIC TENSOR VARIABLES (species by species)
c
c     complex K0s(0:IONMAX),K1s(0:IONMAX),K2s(0:IONMAX),K3s(0:IONMAX),
c    %        K4s(0:IONMAX),K5s(0:IONMAX)
c     common/tensor_cmn/K0s,K1s,K2s,K3s,K4s,K5s
c
c   DIELECTRIC TENSOR VARIABLES (total) + DISPERSION EQUATION SOLUTION
c
c     complex K0,K1,K2,K3,K4,K5,nperp2
c
c  The following sets of constants are defined in subroutine Initcnsts
c  and are of obvious utility everywhere.
c
c   MATHEMATICAL CONSTANTS
c   ----------------------
c
c     real pi,piroot,twopi
c     common/kmath/pi,piroot,twopi
c     data pi/3.1415926535/,piroot/1.772453651/,twopi/6.283185308/
c
c   1986 FUNDAMENTAL CONSTANTS (R.Cohen, B.Taylor,Physics Today,Aug.1991)
c   --------------------------
c
c     real e,me,mp,mu0,eps0,c
c     common /kphysic/e,me,mp,c,mu0,eps0
c     data  e/1.60217733e-19/, c/2.99792458e+ 8/
c     data me/9.1093897 e-31/,mp/1.6726231 e-27/
c
c   ii) *** Public Subroutines ***
c
c    Prior to any call to this package, call Initcnsts to initialise
c    constants necessary to tensor evaluation.
c
c ==>     CALL INITCNSTS()
c
c    To obtain the dielectric tensor for a given nperp2:
c
c ==>     CALL TENSOR(nperp2,K0,K1,K2,K3,K4,K5)
c
c    To obtain the nperp2 of the Fast Wave together with the associated
c    dielectric tensor:
c
c ==>     CALL fastwave(nperp2,K0,K1,K2,K3,K4,K5,nperp2_max)
c
c    To obtain more than one root, a subroutine n_root(nperp2,nroot) is
c    given.
c
c ==>     CALL N_ROOT(nperp2,nroot)
c
c    The number nroot of roots nperp2(i) is limited to a maximum of
c    five. There is no warranty that the subroutine is capable to give
c    five roots in every situation. The fastwave root is fairly robust
c    though. To know the dielectric tensor components for the found roots,
c    you must then call the subroutine tensor separately for each roots,
c    once you have checked the roots validity.
c
c   Internally adjustable parameters :
c   --------------------------------
c
c   - The number of cyclotron harmonics considered in the sum over Bessel
c     functions is NNmax and is currently fixed at 20.
c   - The maximum number of ion species IONMAX is fixed at 10.
c
c
c  ANY OTHER VARIABLES AND SUBROUTINES SHOULD NOT BE INVOKED FROM OUTSIDE
c  OF THIS PACKAGE, AS THEY ARE ONLY INTENDED TO BE MEANINGFUL AND UPDATED
c  IN THE CONTEXT OF THIS PACKAGE. BE ALSO CAREFUL OF NEVER MODIFY OR
c  REDEFINE IN ANY WAY WHATSOEVER THE INTERNAL VARIABLES OF THE PACKAGE, OR
c  DO IT AT YOUR OWN RISKS.
c
c
c=======================================================================
c
c       written by M.P.Evrard ; LPP/ERM-KMS ; Sept 14 1994
c       tested  by M.P.Evrard with the precious help of Dirk Van Eester.
c       double precision version: Feb 17 1995
c       dzanly replacement: March 7 1995
c
c=======================================================================
c
c
c=======================================================================
c   .    1    .    2    .    3    .    4    .    5    .    6    .    7
c=======================================================================
c
      subroutine InitCnsts()
	  implicit none
c
c  LOCAL VARIABLES
c
      real kilo
      parameter(kilo=1000.)
c
c  MATHEMATICAL CONSTANTS
c
      real pi,piroot,twopi
      common/kmath/pi,piroot,twopi
      data pi/3.1415926535/,piroot/1.772453651/,twopi/6.283185308/
c
c 1986 Fundamental Constants (R.Cohen, B.Taylor,Physics Today,Aug.1991)
c
      real e,me,mp,mu0,eps0,c
      common /kphysic/e,me,mp,c,mu0,eps0
      data  e/1.60217733e-19/, c/2.99792458e+ 8/
      data me/9.1093897 e-31/,mp/1.6726231 e-27/
c
c  DIELECTRIC TENSOR CONSTANTS
c
      real kwp2e,kwce,kvte,kwp2i,kwci,kvti
      common /ktensor/kwp2e,kwce,kvte,kwp2i,kwci,kvti
c
      mu0=4.e-7*pi
      eps0=1./(mu0*c**2)
c
      kwp2e=(e*c)**2*mu0/me
      kwce=e/me
      kvte=2.*kilo*e/me
      kwp2i=(e*c)**2*mu0/mp
      kwci=e/mp
      kvti=2.*kilo*e/mp
c
      return
      end
c
c=======================================================================
c   .    1    .    2    .    3    .    4    .    5    .    6    .    7
c=======================================================================
c
      subroutine fastwave(nperp2,K0,K1,K2,K3,K4,K5,nperp2_max)
	  implicit none
c
c   USES IMSL SUBROUTINE dzanlY !!
c   Unfortunately, no NAG equivalent exists.
c
c    OUT/IN VARIABLES
c
      complex nperp2,K0,K1,K2,K3,K4,K5
      real nperp2_max
 
c nperp2_max can be set to an estimate of the max expected nperp2.
c if the root exceeds this value, then this subroutine will attempt
c to find a smaller root.
 
c
c   LOCAL VARIABLES:
c
      integer info(1),nknown,nroot,nguess,itmax
      real*8 errrel,errabs,aux,errconj
      complex*16 zinit(1),zroot(1),dispeqn,rootold
      complex*16 zinits(3)
      complex zroots(3)
      complex*16 K0int,K1int,K2int,K3int,K4int,K5int
      external dispeqn
      data info(1)/0/,nknown/0/,nroot/1/,nguess/1/,itmax/50/
      data errrel,errabs/1.d-7,1.d-7/errconj/1.d-5/
      data zinit(1)/(0.d0,0.d0)/
      save zinit
 
      integer nwarns,mxwarns,ikeep,i
      data nwarns /0/, mxwarns /20/
      real*8 zmagrat2
      save nwarns
 
c GWH: Define the function CABS1 to be a 1-norm of a complex number.
c This will evaluate much faster than CABS which involves taking
c a square root.
c
      complex*16 z1
      real*8 cabs1
      cabs1(z1) = abs(real(z1)) + abs(imag(z1))
c
c   CODE:
c
c************************************************
c
      call dzanly(dispeqn,errabs,errrel,
     %            nknown ,nroot ,nguess,zinit,itmax,zroot,info)
c
c    What do we do if Im(FW)<0.? Try to find if there is a complex
c    conjugate root.
c
      zmagrat2=1.e-2
      if(imag(zroot(1)).lt. -zmagrat2*cabs1(zroot(1))) then
         zinit(1)=conjg(zroot(1))
         rootold=zroot(1)
         call dzanly(dispeqn,errabs,errrel,
     %        nknown ,nroot ,nguess,zinit,itmax,zroot,info)
 
         aux=cabs1(conjg(rootold)-zroot(1))/
     %        (cabs1(rootold)+cabs1(zroot(1)))
c
c    If the new root is not conjugate of the old one, this is not
c    a Conversion Zone, Im(FW) is negative due to temperature
c    anisotropy.==> keep the first root
c
c        if(aux.ge.errconj) then
c            zroot(1)=rootold
c         endif
c
 
c GWH: We are presently running with isotropic temperatures, so anisotropy
c can't be the excuse for unstable waves.  Use the less stringent algorithm
c which Dave Smithe used in his original root.for subroutine.  This
c algorithm has been widely tested in TRANSP and FPP.
 
         if(aux.lt.0.1 .and.imag(rootold)*imag(zroot(1)).le. 0.0)then
c keep the new root
         else
c switch back to the old root
            zroot(1)=rootold
         endif
 
      endif
 
c Retry, if the root seems too big, or is still too unstable:
 
      if(imag(zroot(1)).lt. -zmagrat2*cabs1(zroot(1)) .or.
     %     cabs1(zroot(1)) .gt. 2*nperp2_max) then
         zinit(1)=(0.0,0.0)
         rootold=zroot(1)
         call dzanly(dispeqn,errabs,errrel,
     %        nknown ,nroot ,nguess,zinit,itmax,zroot,info)
 
         ikeep=0
         if(imag(rootold) .ge. 0.0) then
            if(imag(zroot(1)) .ge. 0.0 .and.
     %           cabs1(zroot(1)) .lt. 0.5*cabs1(rootold))
     %           ikeep=1
         endif
 
         if(imag(rootold) .lt. 0.0) then
            if(imag(zroot(1)) .ge. imag(rootold) .and.
     %           cabs1(zroot(1)) .lt. 1.0*cabs1(rootold))
     %           ikeep=1
         endif
 
         if(ikeep .eq. 0) zroot(1)=rootold
 
      endif
 
c
c GWH:  generate a warning message if the root selection algorithm
c may not have worked:
 
c      if(imag(zroot(1)) .lt. -4*zmagrat2*cabs1(zroot(1)) .or.
c     %     cabs1(zroot(1)) .gt. 8*nperp2_max) then
c ignore large roots, but warn about very unstable ones:
      if(imag(zroot(1)) .lt. -4*zmagrat2*cabs1(zroot(1))) then
 
c Another approach would be to search for multiple roots when the first
c root shows signs of problems.  But n_root appears to blow up for some
c cases:
c         nroot=3
c         call n_root(zroots,nroot)
c         nroot=1
 
         nwarns=nwarns+1
         if(nwarns .le. mxwarns) then
c           write(6,*) 'growing or large ROOT warning (usually ignorable):'
c           write(6,*) '2*nperp2*max, zroot(1) =', 2*nperp2_max,zroot(1)
            write(6,*)
     %      'fastwave.for warning: unstable ROOT (usually ignorable):'
            write(6,*) 'zroot(1) =', zroot(1)
         endif
         if(nwarns .eq. mxwarns) then
            write(6,*) '# of fastwave.for ROOT warnings > ',mxwarns
            write(6,*) 'suppressing further warning messages...'
         endif
      endif
 
      call tensor2(zroot(1),K0int,K1int,K2int,K3int,K4int,K5int)
c
      zinit(1)=zroot(1)/2.
      nperp2=zroot(1)
      K0=K0int
      K1=K1int
      K2=K2int
      K3=K3int
      K4=K4int
      K5=K5int
c
c************************************************
c
      return
      end
c
c=======================================================================
c   .    1    .    2    .    3    .    4    .    5    .    6    .    7
c=======================================================================
c
      subroutine n_root(nperp2,nroot)
	  implicit none
c
c   USES IMSL SUBROUTINE dzanlY !!
c   Unfortunately, no NAG equivalent exists.
c
c    IN VARIABLE
c
      integer nroot
c
c    OUT VARIABLES
c
      complex nperp2(*)
c
c   LOCAL VARIABLES:
c
      integer info(5),nknown,nguess,itmax,nmax,i
      real*8 errrel,errabs
      complex*16 zinit(5),zroot(5),dispeqn
      external dispeqn
      data info/5*0/,nknown/0/,nguess/5/,itmax/10/,nmax/5/
      data errrel,errabs/1.d-7,1.d-7/
      data zinit/5*(0.d0,0.d0)/
      save zinit
c
c   CODE:
c
c************************************************
c
      if(nroot.gt.nmax) nroot=nmax
	  do i=1,nroot
	  zinit(i)=(0.d0,0.d0)
	  enddo
      call dzanly(dispeqn,errabs,errrel,
     %            nknown ,nroot ,nguess,zinit,itmax,zroot,info)
      do i=1,nroot
      nperp2(i)=zroot(i)
      zinit(i)=zroot(i)
      enddo
c
c************************************************
c
      return
      end
c
c=======================================================================
c   .    1    .    2    .    3    .    4    .    5    .    6    .    7
c=======================================================================
c
      subroutine tensor(nperp2,K0,K1,K2,K3,K4,K5)
      implicit none
c	
c     Interface to the double precision version tensor2 used internally
c     by the dispersion equation subroutine.
c
c    IN VARIABLES
c
      complex nperp2
c
c    OUT VARIABLES
c
      complex K0,K1,K2,K3,K4,K5
c
c    LOCAL VARIABLES
c
      complex*16 nperp2int
      complex*16 K0int,K1int,K2int,K3int,K4int,K5int
c
c   CODE:
c
c************************************************
c
      nperp2int=nperp2
      call tensor2(nperp2int,K0int,K1int,K2int,K3int,K4int,K5int)
      K0=K0int
      K1=K1int
      K2=K2int
      K3=K3int
      K4=K4int
      K5=K5int
c
c************************************************
c
      return
      end
c
c=======================================================================
c   .    1    .    2    .    3    .    4    .    5    .    6    .    7
c=======================================================================
c
c  Here starts the private part of this package.
c
c=======================================================================
c   .    1    .    2    .    3    .    4    .    5    .    6    .    7
c=======================================================================
c
      subroutine tensor2(nperp2,K0,K1,K2,K3,K4,K5)
      implicit none
c
c  This subroutine is for INTERNAL usage only. It uses double precision
c  to obtain the correct nperp2 and Ki, but outside the dielectric package
c  we only use single precision.
c
c    IN VARIABLES
c
      complex*16 nperp2
c
c    OUT VARIABLES
c
      complex*16 K0,K1,K2,K3,K4,K5
c
c   PLASMA VARIABLES            ! Those variables are coming from
c                               ! outside the package and are thus
c                               ! single precision.
c
      integer nions,IONMAX
      parameter(ionmax=10)
      real npar          ! kpar/k0
      real omega         ! twopi*RF frequency
      real bmag          ! local magnetic field
      real dense         ! electron density [m-3]
      real teper         ! perpendicular electron temperature [keV]
      real tepar         ! parallell electron temperature [keV]
      real awi(IONMAX)   ! ion atomic mass Ai
      real qi(IONMAX)    ! ion charge      Zi
      real densi(IONMAX) ! ion density [m-3]
      real tiper(IONMAX) ! perpendicular ion temperature [keV]
      real tipar(IONMAX) ! parallell ion temperature [keV]
 
      common /plasma_cmn/ npar,omega,bmag,dense,teper,tepar,awi,qi,
     %                    densi,tiper,tipar,nions
c
c   DIELECTRIC TENSOR VARIABLES ! Those variables are transferred
c                               ! outside the package and are thus
c                               ! single precision.
c
      complex K0s(0:IONMAX),K1s(0:IONMAX),K2s(0:IONMAX),K3s(0:IONMAX),
     %        K4s(0:IONMAX),K5s(0:IONMAX)
      common/tensor_cmn/K0s,K1s,K2s,K3s,K4s,K5s
c
c   SWITCH
c
      integer model(0:IONMAX)
      common /switches/model
c
c   LOCAL VARIABLES
c
      integer ion,nspec,Nmax
      complex*16 lambda,kperp2
      complex*16 K0spc,K1spc,K2spc,K3spc,K4spc,K5spc
      real*8 wp2,wc,ksi,vth,alpha,rhoL,lmin
      real   bmin,kpmin,tmin,bloc,kploc,kpar,ak0
      data bmin /0.001/  ! |bmag| > 10 Gauss
      data kpmin/0.001/  ! |kpar| >  1 km-1
      data tmin /0.001/  !  Temp  >  1 eV
      data Nmax /5/
      data lmin/0.0005/
c
c   NUMERICAL CONSTANTS
c
      real e,me,mp,mu0,eps0,c
      common /kphysic/e,me,mp,c,mu0,eps0
c
      real kwp2e,kwce,kvte,kwp2i,kwci,kvti
      common /ktensor/kwp2e,kwce,kvte,kwp2i,kwci,kvti
c
c   CODE:
c
c************************************************
c
      ak0=omega/c
      kpar=npar*ak0
      kperp2=nperp2*ak0**2
c
      bloc =sign(max(abs(bmag),bmin ),bmag)
      kploc=sign(max(abs(kpar),kpmin),kpar)
      K0=(0.d0,0.d0)
      K1=(1.d0,0.d0)
      K2=(0.d0,0.d0)
      K3=(1.d0,0.d0)
      K4=(0.d0,0.d0)
      K5=(0.d0,0.d0)
c
c  i) Electron contributions
c
      wp2=kwp2e*dense
      wc=kwce*bloc
      vth=sqrt(kvte*max(tepar,tmin))
      ksi=omega/(kploc*vth)
      vth=sqrt(kvte*max(teper,tmin))
      lambda=kperp2*(vth/wc)**2/2.
      rhoL=omega*vth/(c*wc)
      wp2=wp2/omega**2
      wc= wc /omega
      alpha=max(teper,tmin)/max(tepar,tmin)
c
      if(model(0).lt.0) then
      call Cold     (K0spc,K1spc,K2spc,K3spc,K4spc,K5spc,
     %                             wp2,wc,ksi,rhoL,lambda,alpha,Nmax)
      elseif(model(0).eq.0) then
      call Truncated(K0spc,K1spc,K2spc,K3spc,K4spc,K5spc,
     %                             wp2,wc,ksi,rhoL,lambda,alpha,Nmax)
      else
        if(abs(lambda).gt.lmin) then
        call Swanson  (K0spc,K1spc,K2spc,K3spc,K4spc,K5spc,
     %                             wp2,wc,ksi,rhoL,lambda,alpha,Nmax)
        else
        call Truncated(K0spc,K1spc,K2spc,K3spc,K4spc,K5spc,
     %                             wp2,wc,ksi,rhoL,lambda,alpha,Nmax)
        endif
      endif
      K0s(0)= K0spc
      K1s(0)= K1spc
      K2s(0)=-K2spc
      K3s(0)= K3spc
      K4s(0)= K4spc
      K5s(0)=-K5spc
      K0=K0+K0spc
      K1=K1+K1spc
      K2=K2-K2spc
      K3=K3+K3spc
      K4=K4+K4spc
      K5=K5-K5spc
c
c  ii) Ion contributions
c
      do ion=1,nions
      wp2=kwp2i*densi(ion)*qi(ion)**2/awi(ion)
      wc=kwci*bloc*qi(ion)/awi(ion)
      vth=sqrt(kvti*max(tipar(ion),tmin)/awi(ion))
      ksi=omega/(kploc*vth)
      vth=sqrt(kvti*max(tiper(ion),tmin)/awi(ion))
      lambda=kperp2*(vth/wc)**2/2.
      rhoL=omega*vth/(c*wc)
      wp2=wp2/omega**2
      wc =wc /omega
	  alpha=max(tiper(ion),tmin)/max(tipar(ion),tmin)
c
      if(model(ion).lt.0) then
      call Cold     (K0spc,K1spc,K2spc,K3spc,K4spc,K5spc,
     %                        wp2,wc,ksi,rhoL,lambda,alpha,Nmax)
      elseif(model(ion).eq.0) then
      call Truncated(K0spc,K1spc,K2spc,K3spc,K4spc,K5spc,
     %                        wp2,wc,ksi,rhoL,lambda,alpha,Nmax)
      else
        if(abs(lambda).gt.lmin) then
        call Swanson  (K0spc,K1spc,K2spc,K3spc,K4spc,K5spc,
     %                             wp2,wc,ksi,rhoL,lambda,alpha,Nmax)
        else
        call Truncated(K0spc,K1spc,K2spc,K3spc,K4spc,K5spc,
     %                             wp2,wc,ksi,rhoL,lambda,alpha,Nmax)
        endif
      endif
      K0s(ion)=K0spc
      K1s(ion)=K1spc
      K2s(ion)=k2spc
      K3s(ion)=K3spc
      K4s(ion)=K4spc
      K5s(ion)=K5spc
      K0=K0+K0spc
      K1=K1+K1spc
      K2=K2+K2spc
      K3=K3+K3spc
      K4=K4+K4spc
      K5=K5+K5spc
      enddo
c
c************************************************
c
      return
      end
c
c=======================================================================
c   .    1    .    2    .    3    .    4    .    5    .    6    .    7
c=======================================================================
c
      subroutine Swanson(K0,K1,K2,K3,K4,K5,wp2,wc,ksi,
     %                                         rhoL,lambda,alpha,Nmax)
      implicit none
c
c   USES IMSL SUBROUTINE DCBINS !!
c   (NAG equivalent: S18DEF)
c
c
c   IN VARIABLES:
c
      integer Nmax
      real*8 wp2         ! wp2    is wp**2/w**2
      real*8 wc          ! wc     is (wc/w)
      real*8 ksi         ! ksi    is w/(kpar*vthpar)
      real*8 rhoL        ! rhoL   is k0*(vthper/wc)
      real*8 alpha       ! alpha  is Tperp/Tpar
      complex*16 lambda  ! lambda is (kperp2 * rhoL**2)/2.
c
c   OUT VARIABLES:
c
c    K0,K1,K2,K3,K4,K5 are as described in Plasma Waves, D.G.Swanson,
c    (4.156ff) p.156,
c        EXCEPT for K0 -> K0/nperp2, K4 and K5 -> K4/nperp,K5/nperp
c
      complex*16 K0,K1,K2,K3,K4,K5
c
c   LOCAL VARIABLES:
c
      integer i,ifail,NNmax
      parameter (NNmax=20)
      real*8 beta,sqrta
      real*8 ksin,fnu,cy(NNmax+2)
      complex*16 cbs(NNmax+2),cbsp(NNmax+1),cbsd(NNmax+1),aux,Im
      complex*16 Zfc(-NNmax:NNmax),Zfcp(-NNmax:NNmax)
      complex*16 Zevn,Zodd,Zpevn,Zpodd,lambd2
      data Im/(0.d0,1.d0)/
      data fnu/0.d0/,ifail/0/
c
c   CODE:
c
c************************************************
c
      if(Nmax.lt.0)     Nmax=-Nmax
      if(Nmax.gt.NNmax) Nmax=NNmax
	  lambd2=lambda
c
c  NAG Users:
      call S18DEF(fnu,lambd2,Nmax+2,'U',cbs,cy,ifail)
c
c  IMSL Users:
c     call dcbins(lambd2,Nmax+2,cbs)
c	
      cbsp(1)=cbs(1)-cbs(2)
      cbsd(1)=(0.d0,0.d0)
      do i=2,Nmax+1
      cbsp(i)=cbs(i)-0.5*(cbs(i-1)+cbs(i+1))
      cbsd(i)=(cbs(i-1)-cbs(i+1))*0.5
      enddo
c
      do i=-Nmax,Nmax
      ksin=ksi*(1.+i*wc)
      call disper(ksin,Zfc(i),Zfcp(i))
      enddo
c
      beta=wc*(1.-alpha)
      sqrta=sqrt(alpha)
c	
      K0=cbsp(1)*Zfc(0)*alpha
      K1=(0.d0,0.d0)
      K2=(0.d0,0.d0)
      K3=-cbs(1)*Zfcp(0)*alpha
      K4=(0.d0,0.d0)
      K5=cbsp(1)*Zfcp(0)*alpha
c
      do i=1,Nmax
      Zevn =alpha*(Zfc(i)+Zfc(-i))-beta*i*(Zfc(i)-Zfc(-i))
      Zodd =alpha*(Zfc(i)-Zfc(-i))-beta*i*(Zfc(i)+Zfc(-i))
      Zpevn=alpha*(Zfcp(i)+Zfcp(-i))-beta*i*(Zfcp(i)-Zfcp(-i))
      Zpodd=alpha*(Zfcp(i)-Zfcp(-i))-beta*i*(Zfcp(i)+Zfcp(-i))
c
      K0=K0+cbsp(i+1)*  Zevn
      K1=K1+cbsd(i+1)*i*Zevn
      K2=K2+cbsp(i+1)*i*Zodd
      K3=K3- cbs(i+1)* (Zpevn+i*wc*Zpodd)
      K4=K4+cbsd(i+1)*  Zpodd
      K5=K5+cbsp(i+1)*  Zpevn
      enddo
c
      aux=exp(-lambda)*ksi*wp2
      K0=K0*aux*rhoL**2
      K1=K1*aux-wp2*(1.-alpha)
      K2=K2*aux*Im
      K3=K3*aux*ksi/alpha
      K4=K4*aux*rhoL/2./sqrta
      K5=K5*aux*rhoL/2./sqrta*Im			
c
c************************************************
c
      return
      end
c
c
c=======================================================================
c   .    1    .    2    .    3    .    4    .    5    .    6    .    7
c=======================================================================
c
      subroutine disper (x,Zfc,Zfcp)
      implicit none
c
c   USES IMSL FUNCTION DDAWS !!
c   (NAG equivalent: S15AFF)
c
c
c   IN VARIABLES:
c
      real*8 x
c
c   OUT VARIABLES:
c
      complex*16 Zfc,Zfcp
c
c   LOCAL VARIABLES:
c
      integer ifail
      real*8 Zreal,Zimag,DDAWS,s15aff
      real*8 xmax,emax
      data xmax/500.d0/,emax/10.d0/
c
c  MATH. CONSTANTS
c
      real pi,piroot,twopi
      common/kmath/pi,piroot,twopi
c
c   CODE:
c
c************************************************
c
c  NAG Users:
      ifail=0
      Zreal=-2.*S15AFF(x,ifail)
c
c  IMSL Users:
c     Zreal=-2.*DDAWS(x)
c	
      Zimag=0.d0
      if(abs(x).lt.emax) Zimag=piroot*exp(-x**2)
      Zfc=cmplx(Zreal,Zimag)
      if(abs(x).lt.xmax) then
      Zfcp=-2.*(1.+x*Zfc)
      else
      Zfcp=cmplx(1./x**2,0.d0)
      endif
c
c************************************************
c
      return
      end
c
c=======================================================================
c   .    1    .    2    .    3    .    4    .    5    .    6    .    7
c=======================================================================
c
      subroutine Truncated(K0,K1,K2,K3,K4,K5,wp2,wc,ksi,
     %                                        rhoL,lambda,alpha,Nmax)
      implicit none
c
c   IN VARIABLES:
c
      integer Nmax	     ! Unused
      real*8 wp2         ! wp2    is wp**2/w**2
      real*8 wc          ! wc     is (wc/w)
      real*8 ksi         ! ksi    is w/(kpar*vth)
      real*8 rhoL        ! rhoL   is k0*rhoL
      real*8 alpha       ! alpha  is Tperp/Tpar
      complex*16 lambda  ! lambda is (kperp2 * rhoL**2)/2.
c
c   OUT VARIABLES:
c
c    K0,....,K5 are as described in Plasma Waves, D.G.Swanson, (4.171 )
c    p.161, except for K0 -> K0/nperp2, K4 and K5 -> K4/nperp,K5/nperp.
c    An error in the expression of K0 has been corrected.
c
      complex*16 K0,K1,K2,K3,K4,K5
c
c   LOCAL VARIABLES:
c
	  integer i,NNmax
      parameter (NNmax=2)
      real*8 beta,sqrta
      real*8 ksin
      complex*16  aux,Im
      complex*16 Zfc(-NNmax:NNmax),Zfcp(-NNmax:NNmax)
      data Im/(0.d0,1.d0)/
c
c   CODE:
c
c************************************************
c
      do i=-NNmax,NNmax
      ksin=ksi*(1.+i*wc)
      call disper(ksin,Zfc(i),Zfcp(i))
      enddo
c
      beta=wc*(1.-alpha)
      sqrta=sqrt(alpha)
c	
      K0=      alpha*Zfc(0)
     %   -0.5*(alpha*(Zfc(1)+Zfc(-1))-beta*(Zfc(1)-Zfc(-1)))
      K1= 0.5*(1.-lambda)*(alpha*(Zfc(1)+Zfc(-1))-beta*(Zfc(1)-Zfc(-1)))
     %   +0.5*lambda*(alpha*(Zfc(2)+Zfc(-2))-beta*2*(Zfc(2)-Zfc(-2)))
      K2=-0.5*((1.-2.*lambda)*(alpha*(Zfc(1)-Zfc(-1))
     %                        -beta *(Zfc(1)+Zfc(-1)))
     %        +lambda*(alpha*(Zfc(2)-Zfc(-2))-beta*2*(Zfc(2)+Zfc(-2))))
      K3=-(1.-lambda)*Zfcp(0)*alpha
     %   -0.5*lambda*((alpha*(Zfcp(1)+Zfcp(-1))-beta*(Zfcp(1)-Zfcp(-1)))
     %           +wc*(alpha*(Zfcp(1)-Zfcp(-1))-beta*(Zfcp(1)+Zfcp(-1))))
      K4= 0.5*(alpha*(Zfcp(1)-Zfcp(-1))-beta*(Zfcp(1)+Zfcp(-1)))
      K5=      alpha*Zfcp(0)
     %   -0.5*(alpha*(Zfcp(1)+Zfcp(-1))-beta*(Zfcp(1)-Zfcp(-1)))
c
      aux=ksi*wp2
      K0=K0*aux*rhoL**2
      K1=K1*aux-wp2*(1.-alpha)
      K2=K2*aux*Im
      K3=K3*aux*ksi/alpha
      K4=K4*aux*rhoL/2./sqrta
      K5=K5*aux*rhoL/2./sqrta*Im			
c
c************************************************
c
      return
      end
c
c=======================================================================
c   .    1    .    2    .    3    .    4    .    5    .    6    .    7
c=======================================================================
c
      subroutine Cold(K0,K1,K2,K3,K4,K5,wp2,wc,ksi,
     %                                       rhoL,lambda,alpha,Nmax)
      implicit none
c
c   IN VARIABLES:
c
      integer Nmax	     ! Unused
      real*8 wp2         ! wp2    is wp**2/w**2
      real*8 wc          ! wc     is (wc/w)
      real*8 ksi         ! Unused
      real*8 rhoL        ! Unused
      real*8 alpha       ! Unused
      complex*16 lambda  ! Unused
c
c   OUT VARIABLES:
c
c    K0,....,K5 are as described in Plasma Waves, D.G.Swanson, (4.171 )
c    p.161, except that we have taken the cold limit
c
      complex*16 K0,K1,K2,K3,K4,K5
c
c   LOCAL VARIABLES:
c
      complex Im
      data Im/(0.d0,1.d0)/
c
c   CODE:
c
c************************************************
c
      K0= (0.d0,0.d0)
      K1= -wp2/(1.-wc**2)
      K2= -Im*wp2*wc/(1.-wc**2)
      K3= -wp2
      K4= (0.d0,0.d0)
      K5= (0.d0,0.d0)
c
c************************************************
c
      return
      end
c
c=======================================================================
c   .    1    .    2    .    3    .    4    .    5    .    6    .    7
c=======================================================================
c
      complex*16 function dispeqn(nperp2)
      implicit none
c
c   IN VARIABLE
c
      complex*16 nperp2
c
c   LOCAL VARIABLES
c
      complex*16 K0,K1,K2,K3,K4,K5,nperp
      complex*16 K(3,3)
c
c   PLASMA VARIABLES
c
      integer nions,IONMAX
      parameter(ionmax=10)
      real    npar,omega,bmag,dense,teper,tepar
      real    awi(IONMAX),qi(IONMAX),densi(IONMAX),tiper(IONMAX),
     %        tipar(IONMAX)
 
      common /plasma_cmn/ npar,omega,bmag,dense,teper,tepar,awi,qi,
     %                    densi,tiper,tipar,nions
 
c
c   CODE:
c
c  Plasma Waves, D.G.Swanson, eqn (4.163) p.157, taking into account
c  the modified definitions of K0,K4,K5, and the errata of the book.
c
c************************************************
c
      nperp=sqrt(nperp2)
      call tensor2(nperp2,K0,K1,K2,K3,K4,K5)
c
      K(1,1)=K1-npar**2
      K(1,2)=K2
      K(1,3)=(K4+npar)*nperp
      K(2,1)=-K2
      K(2,2)=K1-npar**2-nperp2*(1.-K0)
      K(2,3)=-K5*nperp
      K(3,1)=(K4+npar)*nperp
      K(3,2)=K5*nperp
      K(3,3)=K3-nperp2
c
      dispeqn=K(1,1)*K(2,2)*K(3,3)+K(2,1)*K(3,2)*K(1,3)
     %        +K(1,2)*K(2,3)*K(3,1)-K(2,1)*K(1,2)*K(3,3)
     %        -K(3,1)*K(2,2)*K(1,3)-K(1,1)*K(3,2)*K(2,3)
c
c************************************************
c
      return
      end
c
c=======================================================================
c   .    1    .    2    .    3    .    4    .    5    .    6    .    7
c=======================================================================
c
      complex*16 function dispeqn2(nperp2)
      implicit none
c
c   IN VARIABLE
c
      complex*16 nperp2
c
c   LOCAL VARIABLES
c
      complex*16 K0,K1,K2,K3,K4,K5,nperp
      complex*16 Tperp,Tcrss,denom
c
c   PLASMA VARIABLES
c
      integer nions,IONMAX
      parameter(ionmax=10)
      real    npar,omega,bmag,dense,teper,tepar
      real    awi(IONMAX),qi(IONMAX),densi(IONMAX),tiper(IONMAX),
     %        tipar(IONMAX)
 
      common /plasma_cmn/ npar,omega,bmag,dense,teper,tepar,awi,qi,
     %                    densi,tiper,tipar,nions
 
c
c   CODE:
c
c  This is another (equivalent) way to write the dispersion equation
c  To use it instead of dispeqn, simply change dispeqn to dispeqn2 in
c  fastwave and N_Root. Does not make any difference though.
c
c
c************************************************
c
      nperp=sqrt(nperp2)
      call tensor2(nperp2,K0,K1,K2,K3,K4,K5)
c
      K4=K4+npar
      K3=K3-nperp2
      K1=K1-npar**2
      denom=1.-K0-(K5**2+K4**2)/K3
      Tperp=(K1-nperp2*K4**2/K3)/denom
      Tcrss=(K2-nperp2*K4*K5/K3)/denom
c
      dispeqn2=(nperp2-Tperp)-Tcrss*(Tcrss/Tperp)
c
c************************************************
c
      return
      end
c
c=======================================================================
c   .    1    .    2    .    3    .    4    .    5    .    6    .    7
c=======================================================================
c
c This code is an equivalent of the IMSL zanly/dzanly subroutine.
c We developed it to use in Transp, on machine where IMSL is not
c available. There is no equivalent in the NAG library. We have tried
c to mimick as closely as possible the behaviour of IMSL zanly. The
c initial points used by the algorithm are likely to be different
c though. There is also no warranty that the two versions of zanly will
c find the roots in the same order, as they start differently. From
c comparison made between this zanly and IMSLs, it is nevertheless not
c possible to tell which version is best. The Muller correction subrou-
c tine was written by D.W.Ignat of PPPL.
c
c=======================================================================
c   .    1    .    2    .    3    .    4    .    5    .    6    .    7
c=======================================================================
c
      subroutine dzanly (f, errabs, errrel, nknown, nnew, nguess,
     %                  xinit, itmax, x, info)
      implicit none
c
c      IN/OUT VARIABLES
c
      integer    nknown, nnew, nguess, itmax, info(*)
      real*8        errabs, errrel
      complex*16    f, xinit(*), x(*)
      external   f
c
c      LOCAL VARIABLES
c
      logical       found
      integer       i, iter, loop, iplus
      real*8        eps,tiny
      complex*16    fprt, frt, root, zcor(3), fcor(3), qmuller
      data tiny/1.e-7/
c
c   CODE:
c
c************************************************
c
c   INITIALISATION
c
      if(nnew.lt.1) then
	  write(*,*) ' Nnew must be greater than or equal to 1 '
      stop
	  endif
c
      eps = min(errrel,0.01d0)
      do i=1, nknown + nnew
         info(i) = 0
         if (i .gt. (nknown + nguess)) xinit(i) = (0.0d0,0.0d0)
		 x(i)=xinit(i)
      enddo
c
c   START MAIN LOOP
c
      loop = nknown + 1
      do while(loop.le.(nknown + nnew))
c
      iter=0
      found=.false.
	  root=x(loop)
	  zcor(1)=root
	  frt=f(root)
	  fprt=frt
      if (loop .ne. 1) then
         do i=1, loop-1
            fprt = fprt/(root - x(i))
         enddo
      end if
	  fcor(1)=fprt
	  if(abs(frt).lt.errabs.and.abs(fprt).lt.errabs) found=.true.
	  zcor(2)=root+(1.0d0,0.0d0)
	  frt=f(zcor(2))
	  fprt=frt
      if (loop .ne. 1) then
         do i=1, loop-1
            fprt = fprt/(root - x(i))
         enddo
      end if
	  fcor(2)=fprt
	  if(abs(frt).lt.errabs.and.abs(fprt).lt.errabs) found=.true.
	  zcor(3)=root+(0.0d0,1.0d0)
	  frt=f(zcor(3))
	  fprt=frt
      if (loop .ne. 1) then
         do i=1, loop-1
            fprt = fprt/(root - x(i))
         enddo
      end if
	  fcor(3)=fprt
	  if(abs(frt).lt.errabs.and.abs(fprt).lt.errabs) found=.true.
	  if(found) goto 1
c
c   to print root history (when debugging),
c   uncomment the next 4 write statements.
c
c	  write(*,*) zcor(1),fcor(1)
c     write(*,*) zcor(2),fcor(2)
c	  write(*,*) zcor(3),fcor(3)
c
c   start iteration
c
      do iter=1,itmax
	  call muller(zcor,fcor)
	  root=zcor(3)
c
c This check is added, because in some infortunate cases where the
c root attraction basin is very small, we can have f(z(3)) so much
c smaller than f(z(1)) and f(z(2)) that it is an actual zero although
c f(z(3)) is not smaller than errabs. Then the Muller iteration will
c give again z(3) as the new root and at the next step you got two
c copies of the same point and the Muller subroutine crashes!
c And this can happen with any permutation of (1,2,3).
c
      qmuller =  (zcor(3) - zcor(2)) /
     %           (zcor(2) - zcor(1))
      if(abs(qmuller).lt.tiny.or.abs(qmuller+1.).lt.tiny) then
      found = .true.
      go to 1
      endif
c
c
      frt = f(root)
      fprt = frt
      if (loop .ne. 1) then
         do i=1, loop-1
            fprt = fprt/(root - x(i))
         enddo
      end if
	  fcor(3)=fprt
c	  write(*,*) zcor(3),fcor(3)
	  if(abs(frt).lt.errabs.and.abs(fprt).lt.errabs) found=.true.
	  if(abs(root-zcor(2)).lt.eps) found=.true.
	  iplus=0
	  if((.not.found).and.abs(fprt).lt.errabs) then
	  found=.true.
	  iplus=itmax
	  endif
	  if(found) goto 1
	  if(iter.eq.itmax) then
c          write(*,*) ' DZANLY--failure to converge within itmax '//
c     %               'iterations for root no ',loop
	  endif
	  enddo
c
c   end iteration
c
      iter=iter+1
    1 x(loop) = root
      info(loop) = iter + iplus
      loop = loop + 1
c
c   END OF MAIN LOOP
c
	  enddo
c
c************************************************
c
      return
      end
c
c=======================================================================
c   .    1    .    2    .    3    .    4    .    5    .    6    .    7
c=======================================================================
c
      subroutine Muller(Zcor,Fcor)
c
c     Copyright D. W. Ignat, Princeton Plasma Physics Laboratory 1994.
c
c     See Numerical Recipes in Fortran, p 262.
c
c
      implicit none
      complex*16 zcor(3), fcor(3), cone, znew, zprov, fprov
      complex*16 amuller, bmuller, cmuller, dmuller, dplus, qmuller
      data    cone/(1.0d0,0.0d0)/
c
c   start a Muller iteration
c
      qmuller =  (zcor(3) - zcor(2)) /
     %           (zcor(2) - zcor(1))
      amuller = qmuller*               fcor(3) -
     %          qmuller*(cone+qmuller)*fcor(2) +
     %          qmuller**2 *           fcor(1)
      bmuller = (cone+2.*qmuller)*     fcor(3) -
     %          (cone+qmuller)**2*     fcor(2) +
     %           qmuller**2      *     fcor(1)
      cmuller = (cone+qmuller)*        fcor(3)
      dmuller = sqrt( bmuller**2 - 4. * amuller * cmuller )
 
      if (abs(bmuller+dmuller) .ge. abs(bmuller-dmuller)) then
         dplus  = bmuller + dmuller
      else
         dplus  = bmuller - dmuller
      endif
c
      znew = zcor(3) - (zcor(3) - zcor(2)) *
     %                                      2. * cmuller/dplus
c
c  sort the roots so to pick up those closest to the new iterate
c
      if(abs(znew-zcor(2)).lt.abs(znew-zcor(1))) then
	  zprov      =zcor(2)
	  zcor(2)=zcor(1)
	  zcor(1)=zprov
	  fprov      =fcor(2)
	  fcor(2)=fcor(1)
	  fcor(1)=fprov
	  endif
      if(abs(znew-zcor(3)).lt.abs(znew-zcor(2))) then
	  zprov      =zcor(3)
	  zcor(3)=zcor(2)
	  zcor(2)=zprov
	  fprov      =fcor(3)
	  fcor(3)=fcor(2)
	  fcor(2)=fprov
	  endif
	  zcor(3)=znew
c
      return
      end
c
c=======================================================================
c   .    1    .    2    .    3    .    4    .    5    .    6    .    7
c=======================================================================
c
c
c=======================================================================
c=======================================================================
c
c   END OF PACKAGE                        LPP/ERM-KMS ; March 8th 1995
c
c=======================================================================
c=======================================================================
c
 
 
