	subroutine detech
C
C     THIS SUBROUTINE CALCULATES THE DETECTOR SENSITIVITIES
C     FOR THE GIVEN SIGHTLINES AS A FUNCTION OF MINOR
C     RADIUS AND PITCH-ANGLE, AND STORES THEM FOR FPP
C
C	IT USES A FINITE BANANA WIDTH MODEL TO TRANSFORM
C	THE PHYSICAL DETECTION TO DETECTION AT <R> AND AT
C	A ZETA0 DETERMINED FROM MU CONSERVATION AT RMAJ+<R>
c
c 17-May-1989 GWH: upgraded to general geometry.
c Improvements which could still be made include upgrading the
c BMFP calculation to general plasma species (not just 3 species),
C and upgrading to a variable step size so that it goes faster.
c Could also interpolate in pitch angle, as it already interpolates in
C minor radius.
C
! 23-oct-1984 gwh:  Implemented a slightly fancier model for the
! orbit shift of co and counter particles.  Rob's old model was
! valid in the limit vpar/v --> +/- 1, (he used deltr=q v/omci),
! but this produced a big jump in the shift across the trapped-
! passing boundary, since for trapped particles we use the exact
! orbit shift.	My new formula is still only first order in
! deltr/a, but it is more accurate for barely passing particles,
! although it reduces to Rob's equation for strongly passing orbits.
!
! 22-oct-1984 gwh:  Updated to f with arbitary energy grid.  Pitch
!	angle grid must still be uniform
! 20-oct-1984 gwh: took mean free path and sigma v calculations
! out of the inner sightline integral loop.  Run time reduced by
! a factor of 3.
!
! modified by greg hammett july 20, 1984 to interpolate
! between radial shells.
!
!
C     SET UP COMMON BLOCKS
C
	include 'FPPCOM'
 
	real xarray(maxxsi/2),yarray(maxxsi/2)	!used for plotting
	real temp(3),den(3)	!for electrons, ions, and impurities
	real almfp(maxshe) !mean free path on each shell
	real smallc(maxshe)!small correction to <sig v> on each shell
 
	DATA CON1,PI/1.92691111E15,3.141592654E00/
	DATA TWOPI/6.283185308/
 
!function which indexes into the SENS array, giving
! the sensitivity of the CX analyzer looking at energy i
! and sightline j to f on shell k at pitch angle L:
	IND(I,J,K,L)=(I-1+nencx*(J-1+NLINE*(K-1+NSHELL*(L-1))))+1
 
	call finitt(0,0) !make sure we are in alpha mode before typing
			 !error messages.
 
	if(ndnb .gt. 2*maxshe)then
	    idum=2*maxshe
	    write(lsysout,*) 'fatal error:  ndnb exceeds the limit of ',
	1	'2*maxshe=',idum				
	    stop
	endif
 
!if the user hasn't specified a value for nencx (number of energies
! at which to calculate CX flux), calculate a default value, and make
! sure that nencx doesn't exceed array sizes:
	if(nencx .eq. 0)nencx=maxen
! don't exceed ENEREV array:
	if(nencx .gt. maxen)then
	nencx=maxen
	write(lsysout,*) ' Warning: nencx reduced to',nencx
	endif
 
!make sure we have enough room in the SENS array:
	nsens=mxsens/(nline*nshell*maxxsi/2)
	if(nencx .gt. nsens)then
	nencx=nsens
	write(lsysout,*) ' Warning: nencx reduced to ',nencx
	endif
 
!if there is not enough room for anything, say so:
	if(nencx .lt. 1)then
	write(lsysout,*)
     1	' Warning, not enough room in arrays for calculating'
	write(lsysout,*)
     1	' any charge exchange spectra.	Try reducing # of'
	write(lsysout,*) ' of CX detectors to 1.'
	Ldir=-1
	return
	endif
 
c initialize zRaxis:
	call momry(0.0,0.0,0,zRaxis,zY,zJac)
 
! main ion density / electron density:
	DHY=(rZimp-rZeff)/(rZimp-aplasz(2))/aplasz(2)
!nimp/ne:
	DIM=(rZeff-aplasz(2))/((rZimp-aplasz(2))*rZimp)
 
!determine energies at which the CX flux is to be calculated:
! Take the first nencx grid points below elim2:
	itop=0
	do 50 i=maxen,1,-1
	if(enerev(i) .gt. elim2)go to 50
	itop=i
	go to 51
50	continue
51	if(itop .lt. nencx)itop=nencx
	NFENCX=itop+1-nencx	!starting energy index
 
 
C!***********************************************************
C!  Loop over selected energies				    *
C!***********************************************************
	do 100 ien=1,nencx
	E=enerev(NFENCX+ien-1)/1000.	!E in KeV
	v=sqrt(E*con1/Ab)
 
C!calculate mean free path for this energy in each shell:
	do 70 ish=1,nshell
	den(1)=1.0e13*denpro(ish)	!electron density
	den(2)=den(1)*dhy		!hydrogenic density
	den(3)=den(1)*dim		!impurity density
	temp(1)=1.0e-3*tepro(ish)
	temp(2)=1.0e-3*tipro(ish)
	temp(3)=temp(2)
	almfp(ish)=bmfp(e,temp,APlasAMU,ab,den,aplasz,rZbeam,NumPlSp)
!also calculate a small correction due to approximate evaluation
! of <sigma v>cx when the data is analyzed:
	smallc(ish)=sigvbt(E/Ab*1000.,tnupro(ish)/APlasAMU(2),'CX')
	1	    /Psgvcx(E/Ab*1000.)
70	smallc(ish)=smallc(ish)*25*sqrt(3.14159)
!smallc contains the normalizations necessary to be consistent
! with Davis, Mueller, & Keane's conventions.  See the comments
! below, just before the SENS calculations.
C
C     TRACK THE BEAMS INTO THE PLASMA
C
	DO 100 ILIN=1,NLINE
 
C step size in major radius for the sightline integrals (cm):
C some good guesses for how large S can be:
	s=Max(abs(Tans(ilin)),Rinwal)**2*(2./maxxsi)
	1   /Max(abs(Tans(ilin)),Rinwal/maxxsi/2.)
c minimum step size needed is given when zgrids=1.0
	data zgrids /3.0/
	s=Min(s,rmin/nshell)/zgrids
 
	DIR=SIGN(1.0,TANS(ILIN))
	STAN=ABS(TANS(ILIN))
	BEAM=1.0	!initialize attenuation factor factor
!Begin at R = outside edge of the plasma:
	call momry(1.0,0.0,0,zRmax,zy0,zjac)
	ins=(zRmax-stan)/abs(s)+1.0
	s=(zRmax-stan)/ins
 
!Have to integrate along sightline inwards in R, then outwards in R:
	DO 711 KOP=1,2
!unless sightline hits the inner wall:
	if(Stan .lt. Rinwal .and. kop .eq. 2)go to 711
 
	IF(KOP.EQ.1) S=ABS(S)
	IF(KOP.EQ.2) S=-ABS(S)
	PHIPA=0.
	IF(KOP.EQ.2) PHIPA=PI
 
!sightline integral is broken in INS steps:
	DO 700 KIP=1,INS
	IF(KOP.EQ.2) GOTO 153
 
!R is the major radius of the beginning of this interval:
	R=zRmax-S*KIP+S
	GOTO 154
153	R=STAN-S*KIP+S
154	continue
	RT=STAN
	PT= (R-S)*(R-S) - RT*RT
	IF(PT.LT.0.0) PT=0.0
	PT2=R*R-RT*RT
	IF(PT2.LT.0.0) PT2=0.0
!dx = stepsize along the sightline of this interval:
	DX = ABS(SQRT(	 PT2   ) - SQRT(PT))
!Ri = major radius at the center of the interval:
	RI=R-0.5*S
!rl = minor radius of this interval:
	call nbimap(ri,0.0,zxi,zth,ier)
	rl=rmin*zxi
!skip if this is outside the plasma:
	IF(RL.GT.RMIN) GOTO 124
 
!interpolate in minor radius to find neutral density, beam attenuation:
!k is the shell closer in than radius rl, L=K+1 is the shell
! further out.	del is the relative weighting:
	K=INT(RL/RMIN*NSHELL+0.5)
	L=K+1
	IF(L.GT.NSHELL) L=NSHELL
	DEL=RL/RMIN*NSHELL+0.5-K
	IF(K.EQ.0) K=1
!take care to extrapolate dnu properly above the last zone center:
	dnuup=neupro(L)
	if(k .eq. nshell)dnuup=neupro(k)**2/neupro(k-1)
	DNU=EXP(DEL*ALOG(dnuup)+(1.-DEL)*ALOG(NEUPRO(K)))
 
!test out asymmetric nuetral density profile:
	if(lnasym .and. RI .gt. zRaxis) then
	dnu=neupro(1)	 ! n0(R>zRaxis) = n0(zRaxis)
	endif
 
!allow for arbitrary neutral density profile:
	if(ldnb)then
	    dnu=FPINTRP(RI,dnbneu,dnbRmj,ndnb)
	endif
 
	akmfp=del/almfp(L) + (1.-del)/almfp(k)
	svcorr=del*smallc(L) + (1.-del)*smallc(k)
	beam=beam*exp(-dx*akmfp) !attenuation of beam as it comes out
 
!turn off attenuation if desired:
! latten = .t. = include standard signal attenuation
!        = .f. = ignore signal attenuation
	if(.not. latten)beam=1.0
 
C
C	.....DETERMINE <R>, AND ZETA AT RMAJ+<R>, THETA = 0
C	.....USING FINITE BANANA WIDTH MODEL
C
! The coding which used to be here has been moved to subroutine
! orbshift.  This slows down DETECH by a factor of 1.34.  Since the coding
! is identical, the slowdown is due to overhead for calling subroutines,
! and the inability of the compiler to optimize redundant calculations
! inside a loop if they are buried in a subroutine.
! Nevertheless, we will leave this in a subroutine in order to ensure
! that DEPO and DETECH use identical orbit shift models, and to allow for
! orbshift to be upgraded to better orbit shift models without needing to
! change DEPO or DETECH.
 
	zE=E
	zVparV=dir*sTan/Ri
	zR=Ri
	zZ=0.0
	call orbshift(zE,zVparV,zR,zZ,zrav,Zeta0,iunconf)
	if(iunconf .eq. 1) goto 124 !skip unconfined orbits
	rdep=zrav
 
!At this point we have assigned a minor radius rdep that this
! orbit comes from.  To find f at this rdep we interpolate beween
! shells.  Kin is the shell inside of rdep, Kout the shell outside
! of rdep.
	kin=int(rdep/rmin*nshell + 0.5)
	kout=kin+1
	if(kout .gt. nshell)kout=nshell
	weight=rdep/rmin*nshell -kin +0.5
	if(kin .lt. 1)kin=1
 
!determine pitch angle index.  Since a single sightline only
! views co or counter particles, we can cut our pitch angle
! space in half:
	L=LKUPR(zeta0,xsi,maxxsi)
	if(L .lt. maxxsi)then
	    if(zeta0 .gt. .5*(xsi(L)+xsi(L+1)) ) L=L+1
	endif
	if(L .gt. maxxsi/2)L=L-maxxsi/2
 
	indexi=ind(ien,ilin,kin,L)
	indexo=ind(ien,ilin,kout,L)
 
!The svcorr factor in the formula for SENS includes some
! normalization factors. One factor of (2 pi)**1.5 is to
! get the standard normalization defined by Davis, Mueller, & Keane.
! Another factor of 25/(2 pi sqrt(2)) converts Rob's f used
! in the code to the physical f (see SCCF).
!
! CX detector sensitivity at energy ien and sightline ilin,
! to f on shell l with pitch angle k:
	SENS(indexi)=SENS(indexi)+DX*BEAM*DNU*svcorr
	1	*(1.-weight)
	SENS(indexo)=SENS(indexo)+DX*BEAM*DNU*svcorr
	1	*weight
 
C!****************************************************************
C  I NOW INCLUDE THE CHARGE EXCHANGE BETWEEN THE BEAM ION AND THE
C  VOLUME SOURCE NEUTRAL.
C!****************************************************************
	IF(LBeam.EQ.0) GO TO 124  !flag to turn off this option
	SIGNN=1.
	IF(KOP.EQ.2) SIGNN=-1.
	COSPA=RT/(R-S/2.)
	PHIL=PHITNS(ILIN)+DIR*SIGNN*ACOS(COSPA)*NiCur
	IF(PHIL.LT.0.) PHIL=PHIL+TWOPI
	IF(PHIL.GT.TWOPI) PHIL=PHIL-TWOPI
	IF(PHIL.LT.PHIMIN.OR.PHIL.GT.PHIMAX) GO TO 124
	YP=0.
	IPHI=(PHIL-FppPhi1)/DPHI+1.5
	IRN=(R-S/2.-RMAJ1)/DRP+1.5
	IYN=(YP-ELEV1)/DYP+1.5
	IF(Litrav.EQ.0) GO TO 122
	DNUVS=DN0VS(IRN,IYN,IPHI)
	IF(DNUVS.EQ.0.) GO TO 122
	TNUVS=T0VS(IRN,IYN,IPHI)/DNUVS
	s2cxv=psgvcx(e/ab*1000.)
	s3cxv=sigvbt(e/ab*1000.,tnuvs/aplasamu(2)*1000.,'CX')
	SENS(indexi)=SENS(indexi)+DX*BEAM*DNUvs*S3CXV/S2CXV
	1   *25*sqrt(pi)*(1.-weight)
	SENS(indexo)=SENS(indexo)+DX*BEAM*DNUvs*S3CXV/S2CXV
	1   *25*sqrt(pi)*weight
122	CONTINUE
C	I NOW INCLUDE THE CHARGE EXCHANGE BETWEEN THE BEAM ION AND
C	THE PRIMARY BEAM NEUTRALS
	SINPA=SQRT(1.-COSPA*COSPA)
	COSPB=FppRTCEN/(R-S/2.)
	IF(ABS(COSPB).GT.1.E00) COSPB=SIGN(1.E00,COSPB)
	SINPB=SQRT(1.-COSPB*COSPB)
	PHIPB=PI
	IF(NiCur*FppRTCEN.LT.0) GO TO 320
	IF(PHIL.GT.PHIBT) PHIPB=0.E00
	GO TO 330
320	IF(PHIL.LT.PHIBT) PHIPB=0.E00
330	COSX=COSPA*COSPB+SINPA*SINPB*COS(PHIPA-PHIPB)
	DO 310 IENG=1,NKENG1
	W0BEAM=EFULL/IENG
	V0B=SQRT(W0BEAM*CON1/BeamAMU)
	DNPB=DN0B(IRN,IYN,IPHI,IENG)
	VREL2=V*V+V0B*V0B-2.*V*V0B*COSX
	EREL=VREL2*BeamAMU/CON1
	s4cxv=psgvcx(erel/BeamAMU*1000.)
	SENS(indexi)=SENS(indexi)+DX*BEAM*DNpb*S4CXV/S2CXV
	1   *25*sqrt(pi)*(1.-weight)
	SENS(indexo)=SENS(indexo)+DX*BEAM*DNpb*S4CXV/S2CXV
	1   *25*sqrt(pi)*weight
310	CONTINUE
 
C!**********************************************************
C! End of assymetric neutral density correction
C!**********************************************************
 
124	CONTINUE
700	continue	!end of path length integral loop
711	continue	!KOP loop, in and out in R along sightlines
100	continue	!end of loop over all sightlines
110	continue	!end of energy loop
 
	IF(LPloD.NE.1) GOTO 555
c	CALL ISBATCH(IRET)
c	IF (IRET .EQ. 1) GO TO 555
 
C
C     EXAMINE SENS GRAPHICALLY
C
899	WRITE(6,820)
820	FORMAT(' INPUT SIGHTLINE # AND ENERGY # TO EXAMINE SENS,
     >	(0,0) TO EXIT')
	READ(5,*) NANG,NENG
	IF(NANG.EQ.0.AND.NENG.EQ.0) GOTO 555
890	WRITE(6,860)
  860 FORMAT(' WHAT SHELL #?  0 TO EXIT')
	READ(5,*) ishell
	CALL NEWPAG
	IF(ishell.EQ.0) GOTO 850
	DO 870 IJK=1,maxxsi/2
	YARRAY(IJK)=SENS(IND(NENG,NANG,ishell,IJK))
	j=IJK
	if(sign(1.0,Tans(nang)) .gt. 0.0) j=j+maxxsi/2
	xarray(ijk)=xsi(j)
870	continue
	call initt(240)
	CALL BINITT
	call fpgrlab
	CALL NPTS(maxxsi/2)
	CALL DLIMX(-1.0,1.0)
	CALL CHECK(XARRAY,YARRAY)
	CALL DSPLAY(XARRAY,YARRAY)
C	ENCODE(50,910,TEXT1)
	WRITE(TEXT1,910)
910	FORMAT('DETECTOR SENSITIVITY$')
	CALL AGTCGT(750,TEXT1)
C	ENCODE(50,920,TEXT1) ishell,NENG,NANG
	WRITE(TEXT1,920) ISHELL, NENG, NANG
920	FORMAT('SHELL #',I2,3X,'ENERGY #',I2,3X,'SIGHTLINE #',I2,'$')
	CALL AGTCGT(0,TEXT1)
C	ENCODE(50,930,TEXT1)
	WRITE(TEXT1,930)
930	FORMAT('VPAR/V$')
	CALL AGTCHS(TEXT1)
	CALL UPAUSE(0)
	GOTO 890
850	continue
	GOTO 899
555	RETURN
	END
