	subroutine fpprf(f1rad,fnbsours)
 
C**********************************************************************
C                                                                     *
C	THIS PROGRAM SOLVES THE BOUNCE AVERAGED                       *
C	FOKKER-PLANCK EQUATION, AS A FUNCTION OF MINOR RADIUS,        *
C	THEN CALCULATES CHARGE-EXCHANGE SPECTRA                       *
C								      *
C	Originally written by Rob Goldston.  For references on how    *
C	this program works see his Ph.D. thesis (Princeton, 1977).    *
c	In particular, the basic approach of bounce-averaging	      *
C	is described by J.G. Cordey, NF 16 p.499 (1976).	      *
C	The code has been given a cursory description in the          *
C	published literature, see:				      *
C		J.D. Strachan, et. al. NF 21, p.67 (1981),            *
C		D.K. Bhadra, et. al. NF 22, p. 763 (1982)).	      *
C                                                                     *
C	FPPRF is a version of FPP developed by Greg Hammett (gwh)     *
c	which includes a bounce-averaged RF operator.  For a          *
c	description of FPPRF and the new RF physics, see my thesis:   *
c                                                                     *
c	Gregory Wayne Hammett, "Fast Ion Studies of Ion Cylcotron     *
c	Heating in the PLT Tokamak," Princeton University Ph.D.       *
c	Thesis, March 1986.                                           *
c                                                                     *
C**********************************************************************
 
c	For comments on the evolution of FPP through various versions,
c	see the file FPPRF.INF.  A document which describes how to run
c	FPPRF without needing to know its guts is FPPRF.DOC.
 
c	FPPRF uses a conservative differencing scheme described in
c	McCoy, Mirin, and Killeen, CPC 24 p.37 (1981).  That the
c	RF operator should be written in conservative form was
c	first made clear upon reading Kerbel and McCoy's "Bounce-
c	averaged ICRH for Toroidal Devices", 1983 Sherwood meeting.
c
c	FPPRF is always under development and testing.
c
 
C**********************************************************************
C                                                                     *
C	COMMON BLOCKS						      *
C                                                                     *
C**********************************************************************
 
	include 'FPPCOM'
	include 'F1COM'
c if desired, write out a namelist for debugging.  (set idebug=1)
	data idebug /0/
	include 'FPPIN'
 
C**********************************************************************
C                                                                     *
C	Local variables						      *
C                                                                     *
C**********************************************************************
	real zwfast(maxshe)  ! fast ion energy density profile (J/cc)
	real zfptscal(maxshe) ! F.P. times scale of RF   (sec)
	integer imultpro(maxshe) ! internal time step multiplier vs. r
 
	time=tstart !current time
	nrloop=0    !index of number of radial transport steps taken
 
C**********************************************************************
C                                                                     *
C	SET UP PLASMA PARAMETERS				      *
C                                                                     *
C**********************************************************************
 
c many plasma parameters should already be set by now.  INPUT and PROF
c have been called by FPPTOP in the stand-alone version, FPDFLT and
c TRtoFP have been called by the TRANSP version.
 
c if desired, write out a namelist for debugging.  (set idebug=1)
      if(idebug .eq. 1) then
	 open(unit=41,file='fppdbg.nml',status='unknown',
     %      form='formatted', recl=75)
	 write(41,fppin)
	 close(unit=41)
      endif
 
c find the species index of the thermal ion species which corresponds
c to the fast ion which is being Fokker-Planck'ed.
	nthermi=0
	do 11 i=1,nions
	if(i .ne. nfasti .and. nAions(i) .eq. nAions(nfasti)
	1   .and. nZions(i) .eq. nZions(nfasti)) nthermi=i
11	continue
	if(nthermi .eq. 0)nthermi=nfasti
 
C
C
C	CALCULATE PSI PROFILES FOR DEPOSITION, DETECTION, LOSS-CONES
C
	CALL PSICAL
C
c	NEUTRAL BEAM DEPOSITION
C
c--Calculate the source of particles due to neutral beam injection.
c
c The source is stored in array FNBSOURS and is added in during
c each time step in subroutine IMP.  Beams can be ignored by setting
c their power to zero, or setting their times to go off before the
c FPP calculation begins.
 
	if(cxonly)goto 39 !skip deposition if cx calc for old f only
	if(nfpdepsw .ne. 2) then !skip if TRANSP already found FNBSOURS
	call fpdepo(f1rad,fnbsours)
	endif
 
c calculate initial RF power deposition profiles
	if(Prftot .ne. 0.0) call rfprof
 
c stop now if no Fokker-Planck calculations are desired (i.e.,
c if only the Beam and RF deposition information was desired).
	if(tcalc .eq. 0.0)then
	    write(lsysout,*) 'Tcalc=0.0 means no Fokker-Plancking.'
	    write(lsysout,*) 'FPP is now finished.'
	    return
	endif
 
c set default time steps:
	do 13 i=1,nshell
	if( nmulti(i) .ne. 0) then
	    imultpro(i)=nmulti(i)
	else if ( nmulti(1) .ne. 0) then
	    imultpro(i)=nmulti(1)
	else
	    imultpro(i)=1
	endif
13	continue
 
c adjust time step if being called from TRANSP:
 
	if(Ltransp .and. Prftot .ne. 0.0)then
c
c estimate the F.P. time scale, as the RF time scale - 0.50 * the
c pitch angle scattering time scale (the 0.50 is to be on the
c conservative side).
c
	esttscal=1.e30
	do 19 i=1,nshell
c fast ion energy density:
	zwfast(i)=3./2.*drions(i,nfasti)*Trions(i,nfasti)*1.6022e-19
c pitch angle scattering rate:
	zloglam=24.0-alog(drelec(i)**.5/Trelec(i))
	znuii=9.016e-8*nZions(nfasti)**2/nAions(nfasti)**.5/
	1   (1.5*Trions(i,nfasti))**1.5 *rZeff*drelec(i)*zloglam
 
	if(zwfast(i) .gt. 0.0) then
	  znuscale=Pichspec(i,nfasti)/zwfast(i)-0.50*znuii
	else
	  znuscale=-0.50*znuii
	endif
	zfptscal(i)=1.e30
	if(znuscale .gt. 1.e-30) zfptscal(i)=1.0/znuscale
19	if(zfptscal(i) .lt. esttscal) esttscal=zfptscal(i)
 
	write(lsysout,*)'FPP starting at t=',time
	write(lsysout,*)' requested time step =',tcalc
	write(lsysout,*)' Estimated  FP time scale =',esttscal
	write(lsysout,*)' Transp''s Ti time step =',dtfppmin
	izsteps=tcalc/2.0/dtfpp+0.5
 
	data zerror /2.0/
	if(zerror*esttscal .lt. tcalc)then
		tcalc=zerror*esttscal
	endif
	if(tcalc .lt. dtfppmin)then
		tcalc=dtfppmin
	endif
 
	dtfpp=tcalc/2.0/izsteps
	write(lsysout,*)'FPP time step set to ',tcalc
 
c internal FP time steps can vary with radius:
 
	data zerror2 /.02/
	do 23 i=1,nshell
	imultpro(i)=dtfpp/2/zfptscal(i)/zerror2
c reduce the minimum to 1 now that dtfpp is tcalc/4:
	if(imultpro(i) .lt. 1)imultpro(i)=1
	if(imultpro(i) .gt. 200)imultpro(i)=200
23	continue
c use more steps on the pesky central zones:
	imultpro(1)=4*imultpro(1)
	imultpro(2)=4*imultpro(2)
	imultpro(3)=2*imultpro(3)
 
	immin=1.e6
	immax=0
	do 24 i=1,nshell
	if(imultpro(i) .lt. immin)immin=imultpro(i)
24	if(imultpro(i) .gt. immax)immax=imultpro(i)
	write (6,*) 'FPP internal time step range:',dtfpp/immax,
	1 dtfpp/immin
 
	endif
 
C     CALCULATE DETECTOR SENSITIVITIES
C
c--gwh:cx detector calculation is time consuming and
c	can be turned off by setting detector direction=-1
c	in the control file:
 
39	if(Ldir .ne. -1 .and. nline .gt. 0)then
	if(Ldir .ne. 0)call detech
	if(Ldir .eq. 0)call detecv !upgraded to work by Bob Kaita 11/20/86
	endif
 
c calculate radial transport coefficients:
	if(mrloop .gt. 0 .and. (fpdifcon .ne. 0.0 .or.
	1   fpdiftra .ne. 0.0 .or.
	1   xneomult .ne. 0.0 .or.
	1   convmult .ne. 0.0 .or. xripmult .ne. 0.0)) call rdoper
 
c--calculate number of time steps:
 
c for the case where MRLOOP radial diffusion steps are done:
c (the radial diffusion loop is controlled at the very end of FPPRF)
	if(mrloop .gt. 0) then !if there are radial diffusion time steps
	loomax=int( (tcalc-2.*dtFPP)/mrloop/dtFPP/2. + 0.5)
	if(loomax .lt. 1)loomax=1
	if( (loomax*mrloop+2) .gt. maxtim )then
	    loomax=(maxtim-2)/mrloop
	    write(lsysout,42)loomax*mrloop+2
42	    format(' number of time steps being reduced to ',i5,/,
	1	' so that arrays used by MOMS don''t overflow.')
	endif
	endif
 
c for the case of MRLOOP=0 where no radial diffusion is done:
	if(mrloop .eq. 0 .or. LTransp) then
	LOOMAX=INT(TCALC/dtFPP/2.+0.5)
	if(loomax .lt. 1)loomax=1
	if(loomax .gt. maxtim)then
	    loomax=maxtim
	    write(lsysout,41)loomax
41	    format(' number of time steps being reduced to ',i5,/,
	1	' so that arrays used by MOMS don''t overflow.')
	endif
	endif
 
c--adjust nSkipCX so we don't overflow CX spectra arrays:
	nspect=nencx*nline*(loomax/nSkipCX+1)
	if(nspect .gt. mxspec)then
	nSkipCX=loomax/(mxspec/nencx/nline-1)
	write(lsysout,*)
	1   'Warning:  only enough room in arrays to calculate'
	write(lsysout,*) 'CX spectra every',nSkipCX,'''th time step.'
	endif
 
c--initialize time array:
	timmy(0)=tstart
	loomin=0
 
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
!  Return to this point after doing a radial transport step.
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
 
!add to time array:
44	do 45 i=loomin+1,loomax
45	timmy(i)=timmy(loomin)+(i-loomin)*2*dtFPP
 
c--calculate later RF power deposition profiles:
	if(nrloop .gt. 0 .and. prftot .gt. 0.0)call rfprof
 
!insure that a few arrays containing sums get initialized to zero:
	do 50 i=loomin,loomax
	betbm(i)=0.0
	betapar(i)=0.0
	do 40 ireact=1,mxfureac
40	fpfustot(i,ireact)=0.0
50	continue
 
! open the disk file where f is to be saved or read :
	if(lfget)open(unit=36,form='unformatted',status='old')
	if(lfsave)open(unit=35,form='unformatted',status='new')
 
! if CXONLY = .T. then we can skip a lot of time consuming
! fokker-planck stuff and just calculate the CX spectrum.
! Reset a lot of parameters here to do this:
	if(cxonly)then
	timmy(1)=timmy(loomax)
	loomax=1
	dtFPP=timmy(1)/2
	dtreal=dtFPP/nmulti(1)
	dteff=dtFPP
	nSkipCX=1	  !Calculate CX spectra every time step (but there
		  !is only 1 time step).
	if(nplte .ne. 0)nplte=1  ! 1 flux vs E plot unless 0 requested.
	npltt=0	  ! no Flux vs. time plots
	endif
 
c******************************************************
c                                                     *
c	LOOP OVER RADIAL SHELLS                       *
c                                                     *
c******************************************************
C
c--the inner and outer shells of the calculation can be
c	set in the input data file for041.dat.  If they
c	are not set, they default the calculation is done
c	over all shells:
c--set default outer shell to be nshell:
	if(nshsto .le. 0)nshsto=nshell
 
	DO 300 NSHE=nshsta,nshsto
 
	TES=TEPRO(NSHE)
	DES=DENPRO(NSHE)*1.0E13
	TIS=TIPRO(NSHE)
	DENUS=NEUPRO(NSHE)
	tnus=tnupro(nshe)
	rOverR=rminpro(nshe)/Rmajpro(nshe)	! r/R
	nf0err=0 !keep track of f<0 errors in IMP if desired.
 
	time=timmy(loomin)
 
c--dtreal = real time step for adi
c  dteff = dtreal * nmulti = half of effective time step after
c					multiple adi calls
	dtreal=dtFPP/imultpro(nshe)
	dteff=dtFPP
 
c--set dtFPP to the "real" time step for the calls that calculate
c	the time advancement matrices:
	dtFPP=dtreal
 
!
! SET UP COEFFICIENTS
!
! calculate old btime (before compression) for use by COMPRS:
	if(cratio .ne. 1)then
 
c need to uncompress all the geometry information used by SCCF,
c not just rmaj and rmin, to get this to work.
c Until this is upgraded, generate a fatal error:
	call aborter(lsysout,
	1   'SCCF/FPPRF not upgraded for general geometry compression')
 
	zrmajn=rmaj
	zrminn=rmin
	zepn=rOverR
	rmaj=rmaj*cratio
	rmin=rmin*cratio**.5
	rOverR=rOverR/cratio**.5
	call sccf
	rmaj=zrmajn
	rmin=zrminn
	rOverR=zepn
	do 70 j=1,maxxsi
70	btimeo(j)=btime(j)
	endif
 
c--calculate bounce time, bounce average terms in pitch angle operator.
	CALL SCCF
 
c--save bounce time for use in radial transport jacobian, if needed:
	if(mrloop .gt. 0 .and. (fpdifcon .ne. 0.0 .or.
	1   fpdiftra .ne. 0.0 .or.
	1   xneomult .ne. 0.0 .or.
	1   convmult .ne. 0.0 .or. xripmult .ne. 0.0)) call rdjaco
 
c--calculate collision frequencies:
	CALL FPCF
 
c--radial diffusion model:
	call difmod
 
c--set up general conservative Fokker Planck equation to be solved:
	call fpoper
 
c--evaluate ohmic electric field contributions
	call elcf
 
C
C Set up the Loss Cone.
c
C Lossco calculates loss(i,j)=0 for unconfined orbits,
c			     =1 for confined orbits.
c loss(i,j) is used by fstart, imp, moms, and preadi, and so Lossco
c  must preceed these routines.  Lossco must follow psical, ingrid,
c  and sccf (needs njTrap from this last routine).
C
	if(lbador)call lossco
 
c fstart must be called before prepre and rfoper because
c information about the analytic form of f near the origin is needed:
	call fstart
 
c copy from fnbsours (NBI source on all shells) to fnbsour (NBI source
c on this shell), and normalize fnbsour to be a source for f:
	call fpdepnm(f1rad,fnbsours)
 
	if(lfget) read(36)f1 !read f from disk if desired
c or from an array that has undergone radial transport or been saved
c from the last call from TRANSP:
	if(nrloop .gt. 0 .or. (LTransp .and. .not. LTr1st))then
	    do 1111 j=1,maxxsi
	    do 1112 i=0,maxen
		f1(i,j)=f1rad(i,j,nshe)
1112	    continue
1111	    continue
	endif
 
c--evaluate quasilinear RF contributions to the Fokker-Planck
c coefficients:
c
c--call rfoper twice, once for each sign of kpar:
	if(prftot .gt. 0.0 .and. .not. cxonly)then
c	call clock('just before rfoper calls')
		call rfoper
		rkpar=-rkpar
		call rfoper
		rkpar=-rkpar
c	call clock('just after rfoper calls')
c
c symmetrize quasilinear coefficients in the trapped region
c of velocity space, and renormalize to give the same amount of
c power as calculated by the RF wave propagation code:
c
		call symrf
	endif
 
c for RF heating, set up special viewing angles to plot f(E):
	if(prftot .gt. 0.0)then
	if(nJxsist .lt. 1 .or. nJxsist .gt. maxxsi)nJxsist=maxxsi/4
	NiAngls(1)=1			! antiparallel
	NiAngls(2)=maxxsi+1-nJxsist	!resonant pitch angle
	NiAngls(3)=maxxsi/2+1		!perpendicular
	NiAngls(4)=nJxsist		!resonant pitch angle
	NiAngls(5)=maxxsi		!parallel
	endif
 
	if(cratio .ne. 1.0)call comprs	!let compression act on f
 
!Calculate CX flux now if that is all we will do:
	if(cxonly)then
	niTime=1
	time=time+2*dteff
	dtFPP=dteff
!if desired, find the contribution from just 1 shell:
	if(nshcx .eq. 0) call spect
	if(nshcx .eq. nshe)call spect
	go to 300
	endif
 
c--calculate Markov matrices which advance the solution in time:
c--implicit operator splitting:
	call prepre
 
c--evaluate collisional contributions to the time advancing matrices:
c--set up recursive coefficients used in ADI method:
	call preadi
 
c--set up fusion cross sections:
	call furate
 
c******************************************************
c                                                     *
c	LOOP FORWARD IN TIME                          *
c                                                     *
c******************************************************
 
c--find initial moments of f, and initial CX spectrum:
	niTime=loomin
	call moms
c	call fusout ! fusion rates now calculated in moms.
c--beam-beam fusion calculation takes 2 cpu seconds, too long
c	to evaluate every time step.  For now, just evaluate it
c	at the beginning and at the end:
c	if(LBBFuf .ne. 0)call bbfuse
 
	IF(Ldir .ne. -1 .and. nline .gt. 0
	1   .and. MOD(niTime,nSkipCX) .EQ. 0)call spect
 
	DO 100 niTime=loomin+1,LOOMAX
 
C	SET UP FISHBONE LOSSES
	if(LFishMdl .ne. 0)call fish
 
c--each execution of this loop advances f by 2*dtFPP:
	dtFPP=dtreal
 
	do 90 itmul=1,imultpro(nshe)
c--choose to call adi once or imp twice:
c--alternating difference implicit method:
c	call adi
c-implicit operator splitting method:
c--each call to imp advance f by dtFPP, while adi calls advanced
c	f by 2*dtFPP.  so we must call imp twice:
 
c isotropize f for benchmarking with isotropic codes if desired:
	data isotr /0/
	call imp
	if(isotr .eq. 1) call fpiso
	call imp
	if(isotr .eq. 1) call fpiso
90	continue
	dtFPP=dteff
 
c--each moms call takes .09 secs !!
	CALL MOMS
c	call fusout ! fusion reates now calculated in MOMS.
 
!only do plots at the end for now:
!	IF(LPlotA.EQ.1.AND.MOD(niTime,nSkipCX).EQ.0) CALL PLOTA
!	IF(LPlotE.EQ.1.AND.MOD(niTime,nSkipCX).EQ.0) CALL PLOTE
!	if(LPlotF .eq. 1 .and. mod(niTime,nSkipCX) .eq. 0)then
!	call plotf
!	call erase
!	endif
 
!calculate contributions to CX spectrum:
	IF(Ldir .ne. -1 .and. nline .gt. 0
	1   .and. MOD(niTime,nSkipCX) .EQ. 0)call spect
 
c--prcdis has not yet been generalized for arbitrary
c	grids.  Thus we need to map f1(0:maxen,maxxsi) which is
c	calculated by adi, into f(52,52). This is done by ftfmap:
	if(npltf .eq. 0)go to 100
	if(mod(niTime,max(1,loomax/npltf)) .eq. 0)then
		call ftfmap
		call prcdis
	endif
100	continue
 
c******************************************************
c                                                     *
c	END OF TIME LOOP			      *
c                                                     *
c******************************************************
 
!calculate final beam-beam reaction rate
c	if(LBBFuf .ne. 0)then
c	niTime=niTime-1	!niTime comes out of loop 1 higher than we think
c	call bbfuse
c	niTime=niTime+1
c	endif
 
	if(nf0err .gt. 0)
	1   write(lsysout,*)'FPPRF: # of f < 0 errors =',nf0err
 
! save f onto disk:
	if(lfsave)write(35)f1
!or into an array for radial transport:
	do 1113 j=1,maxxsi
	do 1114 i=0,maxen
	    f1rad(i,j,nshe)=f1(i,j)
1114	continue
1113	continue
 
!plot final f if desired:
	if(LPlotA .ne. 0 .and. nrloop .eq. mrloop)call plota
	if(LPlotE .ne. 0 .and. nrloop .eq. mrloop)call plote
	if(LPlotF .ne. 0 .and. nrloop .eq. mrloop)call plotf
 
c--plot surface averaged quantities such as density and temperature
c	and power flow.  Also saves power flow info from the last
c	time step for a later radial power balance plot.
	call plshel(f1rad,fnbsours)
 
300	continue
 
c******************************************************
c                                                     *
c	END OF RADIAL SHELL LOOP		      *
c                                                     *
c******************************************************
 
!if we read f from disk, all we have now is the CX spectrum:
	if(cxonly)then
	call plotss
	return
	endif
 
c Calculate a few quantities of interest for future iterations:
c Update Trions, TrionsPa for the fast ion species:
c and calculate an average kperp-rho correction:
	rkpcorr=0.0
	zrfsum=0.0
	do 1500 i=1,maxshe
	Trions(i,nfasti)=rbtemp1(i)*1000.
	Trionspa(i,nfasti)=rbtpar1(i)*1000.
	rkpcorr=rkpcorr+
	1   qldamp(i)*rkpcpro(i)*Pichspec(i,nfasti)*dVolfp(i)
	zrfsum=zrfsum + Pichspec(i,nfasti)*dVolfp(i)
1500	continue
	if(zrfsum .ne. 0.0) then
	  rkpcorr=rkpcorr/zrfsum
	else
	  rkpcorr=1.0
	endif
 
! plot all kinds of goodies:
! the logic here is that we want one initial and two final plots, one with
! nrloop=mrloop-1 (just before the final radial time step), the 2cd with
! nrloop=mrloop (just after the final radial time step)
! Skip plots if this is the TRANSP version, since TRANSP will make the
! plots for us:
 
	If (.not. LTransp) then
 
	if(nrloop .ge. (mrloop-1) .or. nrloop .eq. 0) then
	    call plotpb !plot final radial power balance
	    call plotfu
	    CALL PLOTMO
	    if(Ldir .ne. -1 .and. nline .gt. 0)call plotss
	endif
 
	endif
 
!Radial Transport loop control:
! MRLOOP is the # of radial diffusion time steps
! MRLOOP=0 is the default for no radial diffusion:
 
	nrloop = nrloop + 1
	if(nrloop .gt. mrloop) go to 9000
	Lfget=.False. !don't read f from disk on future time steps
 
!Do a radial transport step:
	if(fpdifcon .ne. 0.0 .or. xneomult .ne. 0.0 .or.
	1   fpdiftra .ne. 0.0 .or.
	1   convmult .ne. 0.0 .or. xripmult .ne. 0.0)then
	    if(nrloop .eq. 1)call RadNeo !first add in neoclassical D
	    call rdstep(f1rad,fnbsours)
	endif
 
!Do another radial transport step for drift-waves:
	if(fpdifdw .ne. 0.0) call rdstepdw(f1rad,fnbsours)
 
! If this is being run from TRANSP, don't bother to
! do an extra iteration:
	if(LTransp .and. nrloop .eq. mrloop) goto 9000
 
!Define next time chunck to do:
! the logic here is that at the end (when nrloop=mrloop), we want to run
! through the code just once more time step in order to get MOMS to do its
! calculations immediately following a radial diffusion step.
	if(nrloop .lt. mrloop)then
	    istep=loomax-loomin
	    loomin=loomax+1
	    loomax=loomin+istep
	else
	    loomin=loomax+1
	    loomax=loomin+1
	endif
 
!make the time step infinitesimal:
	Timmy(loomin)=Timmy(loomin-1)+dtFPP/100.
	go to 44
 
9000	continue
 
c print a warning message about the convergence if necessary:
	if(prftot .ne. 0.0) then
 
	do i=1, nshell
	if(rbtemp1(i)*1000. .gt. enerev(maxen)/10.)then
	write(lsysout,*)'************************************'
	write(lsysout,*)
     >     'FPP warning:  energy grid might not extend to high enough'
	write(lsysout,*)
     >     'energy to represent this energetic tail which has an'
	write(lsysout,'(1x,a,1pe10.3,a,i3,a)')
     >     'average energy of ',1.5*rbtemp1(i),' keV on shell ',i,'.'
	write(lsysout,'(1x,a,1pe10.3,a)')
     >     'Max energy grid point is only ',enerev(maxen)/1000.,' keV.'
	endif
	enddo
 
	endif
 
	return
	END
