c
c=======================================================================
c   .    1    .    2    .    3    .    4    .    5    .    6    .    7
c=======================================================================
c
c     Orthogonalization of discrete coordinates.  C.W. Davies.
c     ref. in Comp. Phys. Commun. 23 (1981) 427
c
      subroutine ortho(x,y,f,imx,jmx)
c
c Ortho orthogonalizes the mesh described by its cartesian coordinates
c x(i,j),y(i,j) by solving the initial value equation described in
c J. Computational Phys. vol. 39 p.164
c
c The j=const. lines are preserved and the i=const. lines are
c shifted around by four point lagrange interpolation.
c The i-values of the orthogonal points are returned in f(i,j)
c but would not normally be required. f(i,j) is used as workspace.
c
c The boundary conditions apply to closed j=const. lines.
c There is an overlap such that
c point (1,j) is equivalent to point (imx-1,j)
c point (2,j) is equivalent to point (imx,j)
c
c The difference scheme is implicit and a tridiagonal system of
c equations must be solved.
c
      include 'SPRINC'
      parameter (imax=nchi+2)
      dimension x(imx,jmx),y(imx,jmx),f(imx,jmx)
      common /rectws/ xk(imax),xx(imax),yy(imax)
c
      imxm=imx-1
      imxmm=imx-2
c
c set up starting values.
      do 150 i=1,imx
  150 f(i,jmx)=float(i)
c
      call ortho_func(x,y,f,imx,jmx)
      do 300 l=2,jmx
c start from the outside and work in.
      j=jmx+1-l
c
c set up the source vector for invert.
      do 200 i=2,imxm
  200 xk(i)=float(i)+2.*f(i,j)
c
c invert the tridiagonal matrix
      call ortho_invert(f(2,j),xk(2),imxmm)
c periodic boundary conditions
      xk(1)=xk(imx-1)-float(imxmm)
      xk(imx)=xk(1)+float(imxmm)
c
c store the i-values of the orthogonal points in f(i,j)
      call ortho_findxi(f(1,j),f(1,j+1),imx)
c
  300 continue
c
c
c shift the i=constant lines
      call ortho_shifts(x,y,f,imx,jmx)
c
      return
      end
c
c=======================================================================
c   .    1    .    2    .    3    .    4    .    5    .    6    .    7
c=======================================================================
c
      subroutine ortho_func(x,y,f,imx,jmx)
c
c ortho_func calculates the function f in the p.d.e.
c
      dimension f(imx,jmx),x(imx,jmx),y(imx,jmx)
c
      imxm=imx-1
      jmxm=jmx-1
c
      do 1 j=1,jmxm
      do 1 i=2,imxm
      dxdi=x(i+1,j)-x(i-1,j)+x(i+1,j+1)-x(i-1,j+1)
      dydi=y(i+1,j)-y(i-1,j)+y(i+1,j+1)-y(i-1,j+1)
      dxdj=x(i,j+1)-x(i,j)
      dydj=y(i,j+1)-y(i,j)
      f(i,j)=(dxdi*dxdj+dydi*dydj)/(dxdi*dxdi+dydi*dydi)
    1 continue
c
      return
      end
c
c=======================================================================
c   .    1    .    2    .    3    .    4    .    5    .    6    .    7
c=======================================================================
c
      subroutine ortho_invert(f,source,msh)
c
c ortho_invert solves a periodic tridiagonal matrix equation of the form
c    f(i)*u(i-1)+u(i)-f(i)*u(i+1)=source(i)
c the unknowns u(i) are returned in source(i)
c
      include 'SPRINC'
      parameter (imax=nchi+2)
      dimension f(msh),source(msh)
      common /rectws/ xk(imax),xx(imax),yy(imax)
c
      source(1)=source(1)+float(msh)*f(1)
      source(msh)=source(msh)+float(msh)*f(msh)
      wa=-f(msh)
      wb=1.0
      wg=f(msh)
      ww=source(msh)
c
      yy(1)=f(1)
      xx(1)=1.0
c
      mshm=msh-1
c
      do 2 j2=2,mshm
      factor=-f(j2)/xx(j2-1)
      xx(j2)=1.0-f(j2-1)*factor
      yy(j2)=yy(j2-1)*factor
      source(j2)=source(j2)+source(j2-1)*factor
      factor=-wa/xx(j2-1)
      wa=-f(j2-1)*factor
      wb=wb+yy(j2-1)*factor
      ww=ww+source(j2-1)*factor
    2 continue
c
      yy(mshm)=yy(mshm)-f(mshm)
      wa=wa+wg
      factor=yy(mshm)*wa-xx(mshm)*wb
c
      if(abs(factor).lt.1.0e-10) stop 100
c
      source(msh)=(source(mshm)*wa-ww*xx(mshm))/factor
      source(mshm)=(ww*yy(mshm)-source(mshm)*wb)/factor
      sn=source(msh)
      idum=mshm-1
c
      do 3 j3=2,mshm
      source(idum)=(source(idum)-yy(idum)*sn+f(idum)*source(idum+1))/
     +xx(idum)
      idum=idum-1
    3 continue
c
      return
      end
c
c=======================================================================
c   .    1    .    2    .    3    .    4    .    5    .    6    .    7
c=======================================================================
c
      subroutine ortho_findxi(xin,xi,imx)
c
c ortho_findxi interpolates xk(i) to find the values of xk(xi(i))
c and then puts xin(i)=xk(xi(i))
c
      include 'SPRINC'
      parameter (imax=nchi+2)
      dimension xin(imx),xi(imx)
      common /rectws/ xk(imax),xx(imax),yy(imax)
      lmod(i)=((i-1)/imxmm)*imxmm
      iper(i)=i-lmod(i)+lmod(imxm-i)
c
      imxm=imx-1
      imxmm=imx-2
c
      do 5 i=2,imxm
      ii=ifix(xi(i))
      xii=xi(i)-ii
c 4-point lagrange interpolation
      a1=-xii*(xii-1.)*(xii-2.)/6.
      a2=(xii*xii-1.)*(xii-2.)/2.
      a3=-xii*(xii+1.)*(xii-2.)/2.
      a4=xii*(xii*xii-1.)/6.
c
      i1=iper(ii-1)
      i2=iper(ii)
      i3=iper(ii+1)
      i4=iper(ii+2)
      xk1=xk(i1)+lmod(ii-1)-lmod(imxm-(ii-1))
      xk2=xk(i2)+lmod(ii)-lmod(imxm-ii)
      xk3=xk(i3)+lmod(ii+1)-lmod(imxm-(ii+1))
      xk4=xk(i4)+lmod(ii+2)-lmod(imxm-(ii+2))
c
      xin(i)=a1*xk1+a2*xk2+a3*xk3+a4*xk4
c
    5 continue
c
      xin(1)=xi(imx-1)-(imx-2)
      xin(imx)=xi(2)+(imx-2)
c
      return
      end
c
c=======================================================================
c   .    1    .    2    .    3    .    4    .    5    .    6    .    7
c=======================================================================
c
      subroutine ortho_shifts(x,y,xi,imx,jmx)
c
c ortho_shifts interpolates x(i),y(i) to the points x(xi(i))=x(i)
c and y(xi(i))=y(i)
c
      include 'SPRINC'
      parameter (imax=nchi+2)
      dimension x(imx,jmx),y(imx,jmx),xi(imx,jmx)
      common /rectws/ xk(imax),xx(imax),yy(imax)
      iper(i)=i-((i-1)/imxmm)*imxmm
c
      imxm=imx-1
      imxmm=imx-2
c
      do 50 j=1,jmx
      do 5 i=2,imxm
      ii=ifix(xi(i,j))
      xii=xi(i,j)-float(ii)
c 4-point lagrange interpolation
      a1=-xii*(xii-1.)*(xii-2.)/6.
      a2=(xii*xii-1.)*(xii-2.)/2.
      a3=-xii*(xii+1.)*(xii-2.)/2.
      a4=xii*(xii*xii-1.)/6.
c make sure that ii is positive
      ii=ii+9*imxmm
      i1=iper(ii-1)
      i2=iper(ii)
      i3=iper(ii+1)
      i4=iper(ii+2)
c
      xx(i)=a1*x(i1,j)+a2*x(i2,j)+a3*x(i3,j)+a4*x(i4,j)
      yy(i)=a1*y(i1,j)+a2*y(i2,j)+a3*y(i3,j)+a4*y(i4,j)
c
    5 continue
c
      do 6 i=2,imxm
      x(i,j)=xx(i)
      y(i,j)=yy(i)
    6 continue
c
      x(1,j)=xx(imxm)
      y(1,j)=yy(imxm)
      x(imx,j)=xx(2)
      y(imx,j)=yy(2)
c
   50 continue
c
      return
      end
c
c=======================================================================
c   .    1    .    2    .    3    .    4    .    5    .    6    .    7
c=======================================================================
c
