! GWH: This is my own version of the atomic physics subroutines, which
! provide an standalone alternative to TRANSP's routines,and which
! will allow me to model He4-He3 plasmas.  By setting a parameter in
! the HE4COM common block, we can fool FRANTIC into calculating neutral
! density profiles for a Helium plasma, or we let FRANTIC just do its
! usual calculations for a Hydrogenic plasma.  The subroutines in this
! file already exist in the TRANSP CMS library, and so are collected
! together in a file with a different name here.
!
! For a hydrogenic plasma,the zero-temperature beam-target subroutines
! should give identical results to the subroutines already in the
! TRANSP CMS library.  My version of the finite-temperature
! beam-target routine SIGVBT uses simple minded 1.5*Ti corrections.
!
! GWH: The formulas are only valid for a range of energies not including
! zero energy.  In two places, the cross-section is evaluated at a
! minimum of 1 eV.
 
	REAL FUNCTION SIGVBT(ZE1,ZT,ZCODE)
c
c  evaluate various beam-target sigma*v's
c  FPP's standalone version of the TRANSP routine SIGVBT
c  Greg Hammett 23-November-1988
c
C  ARGUMENTS:  INPUT
C
C  "PER NUCLEON" APPLIES FOR ATOMIC REACTIONS
C
C  FOR NUCLEAR REACTIONS THE CALLER SUPPLIES ACTUAL ENERGIES AND
C  TEMPERATURES.
C
C   ZE1-- ENERGY in eV (PER NUCLEON?) OF SINGLE PARTICLE ENTERING
C       A *MAXWELLIAN* DISTRIBUTION OF ANOTHER SPECIES OF PTCLS,
C       WITH WHICH THE SINGLE PARTICLE MAY UNDERGO REACTIONS
C
C   ZT--- TEMPERATURE in eV (PER NUCLEON?) OF THE MAXWELLIAN POPULATION
C       INTO WHICH THE SINGLE PTCL IS ENTERING.  THE ENERGY ZE1 OF
C       THAT SINGLE PTCL IS WITH RESPECT TO THE REST FRAME OF THE
C       MAXWELLIAN POPULATION
C
C  ZCODE -- CHARACTER CODE IDENTIFYING THE DESIRED DATUM
C
C  ATOMIC -- ENERGY AND TEMPERATURE INPUTS PER NUCLEON
C
C   "CX" -- CHARGE EXCHANGE
C   "XE" -- CHARGE EXCHANGE AVG ENERGY OF REAGENT SAMPLED FROM MAXWELLIAN
C   "XF" -- CHARGE EXCHANGE AVG RELATIVE MOMENTUM OF REAGENT SAMPLED
C           I.E. < VOUT DOT VIN / |VIN|*|Vth| > FROM MAXWELLIAN
C   "II" -- IMPACT IONIZATION
C
C  NUCLEAR -- INPUT ACTUAL ENERGIES AND TEMPERATURES
C
C   "DD" -- D->D FUSION
C   "DT" -- D->T FUSION
C   "TD" -- T->D FUSION
C   "TT" -- T->T FUSION
c   "D3" -- D->He3 Fusion
c   "3D" -- He3->D Fusion
c   "T3"
c   "3T"
C
C
C
C    OUTPUT:  SIGVBT  EITHER A REACTION RATE COEFFICIENT
C      (SIGMA*V) WHICH WHEN MULTIPLIED BY THE DENSITIES OF EACH
C      OF THE REAGENT POPULATIONS RESULTS IN A REACTION RATE
C      (#/CM3/SEC)  UNITS--CM**3/SEC
C        OR AVG ENERGY-- EV/NUCLEON
C
C
C----
C
C
C
	include 'FPPCOM'
	CHARACTER*(*) ZCODE
C
C-------------------------------
C
	zerel=(ze1+1.5*zt)
 
	IF(ZCODE.EQ.'CX') THEN
	zans=psgvcx(zerel)
 
	ELSE IF(ZCODE.EQ.'XE') THEN
	call aborter(6,'standalone SIGVBT has no "XE"')
 
	ELSE IF(ZCODE.EQ.'XF') THEN
	call aborter(6,'standalone SIGVBT has no "XF"')
 
	ELSE IF(ZCODE.EQ.'II') THEN
	zans=psgvii(zerel)
 
	ELSE IF(ZCODE.EQ.'DD') THEN
 
c calculate the D-->D fusion reaction sigma*v, using a simple
c estimate of the finite Ti correction to the relative energy:
	zerelnuc=zE1/2+zT/2
	zerel=zerelnuc*2
	zans=sigma(4,0,6,zerel*1.e-3)*1.e4*sqrt(zerelnuc*1.9158e12)
 
c--if desired, use Rob Goldston's formulas which include finite Ti:
c--these formulas good only for d-d:
 
	if(fufiti .eq. 1.0) then
	zzE1=zT*1.5E-3
	zzE2=zE1/1000.0
	zENREL=(zzE1**3+zzE2**3+6.0*zzE1*zzE2*(zzE1+zzE2))**(1./3.)
	zVREL=1.384E6*SQRT(zENREL*1000./2.)
c--this formula for the cross-section is old! It's out of
c	Glasstone and Lovberg.
	zans=144./zENREL*EXP(-45.8/SQRT(zENREL))*1.0E-24*zVREL
	endif
 
	ELSE IF(ZCODE.EQ.'DT') THEN
 
c calculate the D-->T fusion reaction sigma*v, using a simple
c estimate of the finite Ti correction to the relative energy:
	if(fufiti .eq. 1.0) then
	zerelnuc=zE1/2+zT/3
	else
	zerelnuc=zE1/2
	endif
 
	zerel=zerelnuc*2
	zans=sigma(1,0,6,zerel*1.e-3)*1.e4*sqrt(zerelnuc*1.9158e12)
 
	ELSE IF(ZCODE.EQ.'TD') THEN
 
c calculate the T-->D fusion reaction sigma*v, using a simple
c estimate of the finite Ti correction to the relative energy:
	if(fufiti .eq. 1.0) then
	zerelnuc=zE1/3+zT/2
	else
	zerelnuc=zE1/3
	endif
 
	zerel=zerelnuc*3
	zans=sigma(1,1,6,zerel*1.e-3)*1.e4*sqrt(zerelnuc*1.9158e12)
 
	ELSE IF(ZCODE.EQ.'3D') THEN
 
c calculate the He3-->D fusion reaction sigma*v, using a simple
c estimate of the finite Ti correction to the relative energy:
	if(fufiti .eq. 1.0) then
	zerelnuc=zE1/3+zT/2
	else
	zerelnuc=zE1/3
	endif
 
	zerel=zerelnuc*3
	zans=sigma(2,1,6,zerel*1.e-3)*1.e4*sqrt(zerelnuc*1.9158e12)
 
	ELSE IF(ZCODE.EQ.'D3') THEN
 
c calculate the D-->He3 fusion reaction sigma*v, using a simple
c estimate of the finite Ti correction to the relative energy:
	if(fufiti .eq. 1.0) then
	zerelnuc=zE1/2+zT/3
	else
	zerelnuc=zE1/2
	endif
 
	zerel=zerelnuc*2
	zans=sigma(2,0,6,zerel*1.e-3)*1.e4*sqrt(zerelnuc*1.9158e12)
 
	ELSE IF(ZCODE.EQ.'TT') THEN
 
c calculate the T-->T fusion reaction sigma*v, using a simple
c estimate of the finite Ti correction to the relative energy:
	if(fufiti .eq. 1.0) then
	zerelnuc=zE1/3+zT/3
	else
	zerelnuc=zE1/3
	endif
 
	zerel=zerelnuc*3
	zans=sigma(5,0,6,zerel*1.e-3)*1.e4*sqrt(zerelnuc*1.9158e12)
 
	ELSE IF(ZCODE.EQ.'T3') THEN
 
c calculate the T-->He3 fusion reaction sigma*v, using a simple
c estimate of the finite Ti correction to the relative energy:
	if(fufiti .eq. 1.0) then
	zerelnuc=zE1/3+zT/3
	else
	zerelnuc=zE1/3
	endif
 
	zerel=zerelnuc*3
	zans=sigma(6,1,6,zerel*1.e-3)*1.e4*sqrt(zerelnuc*1.9158e12)
 
	ELSE IF(ZCODE.EQ.'3T') THEN
 
c calculate the He3-->T fusion reaction sigma*v, using a simple
c estimate of the finite Ti correction to the relative energy:
	if(fufiti .eq. 1.0) then
	zerelnuc=zE1/3+zT/3
	else
	zerelnuc=zE1/3
	endif
 
	zerel=zerelnuc*3
	zans=sigma(6,0,6,zerel*1.e-3)*1.e4*sqrt(zerelnuc*1.9158e12)
 
	ELSE
	    call aborter(6,'SIGVBT-- unanticipated value of "ZCODE"')
	ENDIF
 
	SIGVBT = ZANS
 
	RETURN
	END
 
	real function psgvcx(zerel)
c calculates charge-exchange sigma*v (cm**3/sec) vs. Energy (eV)
	psgvcx=sigcx(zerel/1000.,1.)*sqrt(zerel*1.9297e12)
	return
	end
 
	real function psgvii(zerel)
c calculates impact-ionization sigma*v (cm**3/sec) vs. Energy (eV)
	psgvii=sigion(zerel/1000.,1.)*sqrt(zerel*1.9297e12)
	return
	end
 
	real function psgvls(zerel,Zimp)
c calculates total electron loss sigma*v (cm**3/sec) vs. Energy (eV)
	psgvls=siglos(zerel/1000.,1.,Zimp)*sqrt(zerel*1.9297e12)
	return
	end
 
	real function psgvei(zTe)
c calculates electron-impact ionization sigma*v (cm**3/sec) vs. Te (eV)
	psgvei=svel(zTe/1000.)
	return
	end
 
	FUNCTION SIGCX(EREL,ANEUT)
C
C  3/28/79
C  THIS FUNCTION WAS WRITTEN BY HARRY H. TOWNER OF THE PRINCETON
C  PLASMA PHYSICS LAB.	THIS FUNCTION WILL CALCULATE THE CHARGE
C  EXCHANGE CROSS SECTION AS GIVEN BY EQ. 4 OF RIVIERE'S PAPER.
C
C  PARAMETER LIST:
C  EREL	  - THE RELATIVE ENERGY BETWEEN THE NEUTRAL AND THE CHARGED
C           PARTICLE.  (KEV)
C	    EG. WNEUT+ANEUT*1.5*TI/zAPLASMA.
C  ANEUT  - THE ATOMIC WEIGHT OF THE NEUTRAL PARTICLE.
C
 
	include 'HE4COM'
	parameter (iNTab=15)
	real sigtab(iNTab,2)	!table of cx cross-sections
	real entab(iNTab,2)	!evaluated at these energies
 
!CX cross-sections from ORNL-5206-VI, p. A.4.38
! for He++ + He0 --> He0 + He++
! and p. A.4.6 for H+ + He0 --> H0 + He+
 
	data entab/.06,.1,.2,.4,.7,
	1	10.,	20.,	50.,	75.,	100.,
	1	200.,	500.,	750.,	1000.,	1400., ! He/He table end
	1	.2,.5,1.,2.,5.,10.,20.,50.,100.,200.,500.,
	1	1000.,2000.,5000.,10.e3/ ! H/He table end
 
	data sigtab/4.0,3.2,2.6,2.2,2.0,
	1	1.6,1.5,1.2,1.1,.9,
	1	.4,.051,.0095,.0026,.00036, ! He/He table end
	1	.015,.029,.052,.1, .31, 1.0, 2.1, 1.1, .3, .033,
	1	8.3e-4,3.6e-5,1.1e-6,8.5e-9,1.4e-10 / !H/He table end
 
	sigcx=0.0 !default
c
c H+ + H0 --> H0 + H+
c
	if(aZneut .eq. 1 .and. aZmain .eq. 1) then
C  zEE IS THE RELATIVE ENERGY (IN EV) PER AMU.
	zEE=max(1.0,EREL*1.E3/ANEUT)
C  THE CROSS-SECTION IS NOW EVALUATED.
	AL10=ALOG10(zEE)
	SIGCX=(1.6666E-16*AL10-2.1505E-15)*AL10+.6937E-14
	SIGCX=SIGCX/(1.E00+.1112E-14*zEE**3.3)
	RETURN
	endif
c
c H+ + He0 --> H0 + He+
c
	if(azneut .eq. 1. and. azmain .eq. 2.)then
	zee=erel/aneut
	SIGCX= 1.0e-16 * fpintrp(zee,sigtab(1,2),entab(1,2),iNTab)
	return
	endif
 
c
c He++ + He0 --> He0 + He++
c
	if(azneut .eq. 2. and. azmain .eq. 2.)then
 
c relative energy (KeV/Amu):
	zee=erel/aneut
	SIGCX= 1.0e-16 * fpintrp(zee*4.,sigtab,entab,iNTab)
	return
	endif
 
	return
	END
 
	FUNCTION SIGION(EREL,ANEUT)
C
C  1/28/80
C  THIS FUNCTION WAS WRITTEN BY HARRY H. TOWNER OF THE PRINCETON
C  PLASMA PHYSICS LAB.	THIS FUNCTION WILL CALCULATE THE PROTON
C  IONIZATION CROSS SECTION AS GIVEN BY FREEMAN AND JONES IN
C  TABLE 2.
C
C  PARAMETER LIST:
C  EREL   - THE RELATIVE ENERGY BETWEEN THE NEUTRAL PARTICLE
C           AND THE CHARGED PARTICLE.  (KEV)
C	    EG. WNEUT+ANEUT*1.5*TI/zAPLASMA.
C  ANEUT  - THE ATOMIC WEIGHT OF THE NEUTRAL PARTICLE.
C
C  SIGION HAS UNITS OF CM**2.
C
	include 'HE4COM'
! Cross-sections for He++ + He0 --> He+ + He+ from ORNL 5206-V I,
! page A.4.38:
	parameter (iNTab=17)
	real sigtab(iNTab),entab(iNTab)
	data entab/
	1	.7,	1.,	2.,	5.,	7.5,
	1	10.,	20.,	50.,	75.,	100.,
	1	200.,	500.,	750.,	1000.,	1400.,
	1	3000.,	6800./
	data sigtab/
	1	.13,	.16,	.21,	.32,	.39,
	1	.45,	.71,	1.8,	2.7,	3.3,
	1	2.4,	.7,	.33,	.2,	.06,
	1	.003,	.00073/
 
	DATA A0,A1,A2,A3,A4,A5,A6/-.4203309E2,.3557321E1,-.1045134E1,
     >.3139238E00,-.7454475E-1,.8459113E-2,-.3495444E-3/
	DATA EXP1/2.718281828E00/
 
	if(aZmain .eq. 2.) go to 50
C
C  EE IS THE RELATIVE ENERGY (IN KEV) PER AMU.
	zEE=max(.001,EREL/ANEUT)
	ALE=ALOG(zEE)
C  THE CROSS-SECTION IS NOW EVALUATED.
	SIGION=EXP1**((((((A6*ALE+A5)*ALE+A4)*ALE+A3)*ALE+A2)*ALE+A1)
     >*ALE+A0)
	RETURN
 
!He4 plasmas:
50	SIGION=1.e-16 * fpintrp(4*erel/aneut,sigtab,entab,iNTab)
	return
	END
 
	FUNCTION SIGLOS(EREL,ANEUT,ZIMP)
C  3/29/79
C  THIS FUNCTION WAS WRITTEN BY HARRY H. TOWNER OF THE PRINCETON
C  PLASMA PHYSICS LAB.  THIS FUNCTION WILL CALCULATE THE CROSS
C  SECTION FOR ELECTRON LOSS (SUM OF CHARGE EXCHANGE AND
C  ION IMPACT IONIZATION) FOR CHARGED PARTICLES COLLIDING
C  WITH ATOMIC HYDROGEN.  FOR ZIMP .GT. 2 THE FORMULA
C  IS BASED ON OLSON ET. AL. PHYSICAL REVIEW LETTERS
C  VOL 41 NUM 3 P. 163.  FOR ZIMP .EQ. 1 THE CHARGE EXCHANGE
C  AND ION IMPACT IONIZATION CROSS-SECTIONS FROM RIVIERE ARE
C  USED.
c  For Zimp=2 and aZneut=2, uses He+He cross-sections.
c  For Zimp=2 and aZneut=1, uses H0 + He++ cross-sections.
c  The H0 + He++ --> H+ + He+ data comes from
c  ORNL-5206-V1, p. A-4-36, (Atomic Data for Controlled Fusion Research,
c  C.F. Barnett ed., 1977) and IPPJ-AM-9 (Kazuhiko Okuno, 1978) are used.
c  There appears to be some serious disagreement in the data below
c  24 keV.
c
C  THE VALUE OF THE CROSS-SECTION IS RETURNED
C  BY SIGLOS AND HAS UNITS OF CM**2.
C
C  PARAMETERS:
C  EREL   - THE RELATIVE ENERGY BETWEEN THE NEUTRAL PARTICLE AND
C           THE IMPURITY.  (KEV)
C	    EG. WNEUT+ANEUT*1.5*TI/zAPLASMA.
C  ANEUT  - THE ATOMIC MASS OF THE NEUTRAL PARTICLE.  (AMU)
C  ZIMP   - THE CHARGE STATE OF THE IMPURITY.
C
	include 'HE4COM'
 
	parameter (intab=9)
	real sigtab(intab)  ! table of cx cross-sections
	real entab(intab)   ! evaluated at these energies
 
c this is a rough digitization of the data, should eventually
c find better data and reduce interpolation errors by giving it
c a finer energy grid:
	data  entab/16.,24.,40.,100.,240.,400.,680.,880.,1200./
	data sigtab/3.5e-16,5.2e-16,9.5e-16,1.6e-15,1.6e-15,1.1e-15,
	1   5.5e-16,2.5e-16,1.4e-16/
 
	IF(ZIMP.LT.1.01E00) GO TO 30
	if(zimp .eq. 2. .and. aZneut .eq. 2.)goto 30
	if(zimp .eq. 2. .and. aZneut .eq. 1.) then
	    siglos=fpintrp(erel/aneut,sigtab,entab,intab)
	    return
	endif
 
! Impurity CX is usually not too big an effect.  For lack of an
! alternative, use the same expression for Helium plasmas too:
C  I NOW USE THE FORMULA FROM OLSON.
	CON1=EREL/(32.E00*ZIMP*ANEUT)
	IF(CON1.GT.87.E00) GO TO 10
	CON2=1.E00-EXP(-CON1)
	GO TO 20
10	CON2=1.E00
20	SIGLOS=4.6E-16*ZIMP*CON2/CON1
	RETURN
30	SIGLOS=SIGCX(EREL,ANEUT)+SIGION(EREL,ANEUT)
	RETURN
	END
 
	FUNCTION SIGVRC(TE)
C  5/15/80
C  THIS FUNCTION WAS WRITTEN BY HARRY H. TOWNER AND WILL CALCULATE
C  <SIGMA*V> FOR RECOMBINATION OF HYDROGEN BY ELECTRONS.  SEE
C  PAPER BY Yu. S. Gordeev, et al. JETP Lett., Vol. 25 No. 3
C  Page 204.
C  PARAMETERS:
C  TE     - THE ELECTRON TEMPERATURE.  (KEV)
C
! Recombination usually is not dominant.  Leave this the same
! even for a Helium plasma.
	IF(TE.GT..4E00) GO TO 100
	SIGVRC=1.48E-14/(SQRT(TE)*(1.E00+4.34E1*TE))
	RETURN
100	SIGVRC=3.58E-16/TE**1.388
	RETURN
	END
 
	FUNCTION SVEL(TEMPE)
C  THIS ROUTINE WAS WRITTEN BY HARRY H. TOWNER OF THE PRINCETON
C  PLASMA PHYSICS LAB.  THIS ROUTINE WILL CALCULATE <SIGMA*V>
C  FOR ELECTRON IMPACT IONIZATION BY USING THE FIT GIVEN BY
C  FREEMAN (CLM-R137 TABLE 3).
C
	include 'HE4COM'
	DIMENSION COEF(7,2)
C
	DATA COEF/-3.173850E1,1.143818E1,-3.833998E00,7.046692E-1,
	1	-7.431486E-2,4.153749E-3,-9.486967E-5,
	1	-.4450917e2,	.2442988e2,	-.1025714e2,
	1	.2470931e1,	-.3426362,	.25051e-1,
	1	-.7438675e-3/
	DATA EXP1/2.718281828E00/
C  COEF ARE THE COEFFICIENTS USED TO MAKE THE FIT TO <SIGMA*V>
	j=1
	if(aZmain .eq. 2.)j=2
	ALN=ALOG(TEMPE*1.E3)
	SVEL=COEF(1,j)+ALN*(COEF(2,j)+ALN*(COEF(3,j)+ALN*(COEF(4,j)+
     >ALN*(COEF(5,j)+ALN*(COEF(6,j)+ALN*COEF(7,j))))))
	SVEL=EXP1**SVEL
	RETURN
	END
